/*
 * wxcdc.cc
 * Command processing: device context
 *
 */

// For compilers that support precompilation, includes "wx.h".
#include "wx_prec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx.h"
#endif

#include "wx_hash.h"
#include "wx_types.h"
#ifdef wx_msw
#include "wx_mf.h"
#endif

#include <iostream.h>
#include <ctype.h>
#include <stdlib.h>

#include "wx_cmds.h"
#include "wxcmdsp.h"

#ifdef USE_CLIPS
#ifndef __WATCOMC__
#include "clips.h"
#endif
#endif


/*
 * Device context functions
 */

#ifdef USE_CLIPS
double clipsDCGetWidth()
{
  if (ArgCountCheck("clipsDCGetWidth", EXACTLY, 1) == -1)
    return 0;

  long id = RtnLong(1);

  return wxcDCGetWidth(id);
}

double clipsDCGetHeight()
{
  if (ArgCountCheck("clipsDCGetHeight", EXACTLY, 1) == -1)
    return 0;

  long id = RtnLong(1);

  return wxcDCGetHeight(id);
}

double clipsDCGetMinX()
{
  if (ArgCountCheck("clipsDCGetMinX", EXACTLY, 1) == -1)
    return 0;

  long id = RtnLong(1);

  return wxcDCGetMinX(id);
}

double clipsDCGetMinY()
{
  if (ArgCountCheck("clipsDCGetMinY", EXACTLY, 1) == -1)
    return 0;

  long id = RtnLong(1);

  return wxcDCGetMinY(id);
}

double clipsDCGetMaxX()
{
  if (ArgCountCheck("clipsDCGetMaxX", EXACTLY, 1) == -1)
    return 0;

  long id = RtnLong(1);

  return wxcDCGetMaxX(id);
}

double clipsDCGetMaxY()
{
  if (ArgCountCheck("clipsDCGetMaxY", EXACTLY, 1) == -1)
    return 0;

  long id = RtnLong(1);

  return wxcDCGetMaxY(id);
}

double clipsDCGetTextExtentWidth()
{
  if (ArgCountCheck("clipsDCGetTextExtentWidth", EXACTLY, 2) == -1)
    return 0.0;

  long id = RtnLong(1);
  char *s = RtnLexeme(2);
  if (!s)
    return 0.0;

  float width = 0.0;
  float height = 0.0;
  wxcDCGetTextExtent(id, s, &width, &height);
  return (double)width;
}

double clipsDCGetTextExtentHeight()
{
  if (ArgCountCheck("clipsDCGetTextExtentHeight", EXACTLY, 2) == -1)
    return 0.0;

  long id = RtnLong(1);
  char *s = RtnLexeme(2);
  if (!s)
    return 0.0;

  float width = 0.0;
  float height = 0.0;
  wxcDCGetTextExtent(id, s, &width, &height);
  return (double)height;
}

long clipsDCOk()
{
  if (ArgCountCheck("clipsDCOk", EXACTLY, 1) == -1)
    return 0;

  long id = RtnLong(1);

  return (long)wxcDCOk(id);
}

long clipsDCBeginDrawing()
{
  if (ArgCountCheck("clipsDCBeginDrawing", EXACTLY, 1) == -1)
    return 0;

  long id = RtnLong(1);

  return (long)wxcDCBeginDrawing(id);
}

long clipsDCEndDrawing()
{
  if (ArgCountCheck("clipsDCEndDrawing", EXACTLY, 1) == -1)
    return 0;

  long id = RtnLong(1);

  return (long)wxcDCEndDrawing(id);
}

long clipsDCSetPen()
{
  if (ArgCountCheck("clipsDCSetPen", EXACTLY, 2) == -1)
    return 0;

  long id = RtnLong(1);
  long penId = RtnLong(2);

  return (long)wxcDCSetPen(id, penId);
}

long clipsDCSetBrush()
{
  if (ArgCountCheck("clipsDCSetBrush", EXACTLY, 2) == -1)
    return 0;

  long id = RtnLong(1);
  long brushId = RtnLong(2);

  return (long)wxcDCSetBrush(id, brushId);
}

long clipsDCSetBackground()
{
  if (ArgCountCheck("clipsDCSetBackground", EXACTLY, 2) == -1)
    return 0;

  long id = RtnLong(1);
  long brushId = RtnLong(2);

  return (long)wxcDCSetBackground(id, brushId);
}

long clipsDCSetFont()
{
  if (ArgCountCheck("clipsDCSetFont", EXACTLY, 2) == -1)
    return 0;

  long id = RtnLong(1);
  long fontId = RtnLong(2);

  return (long)wxcDCSetFont(id, fontId);
}

long clipsDCSetColourMap()
{
  if (ArgCountCheck("clipsDCSetColourMap", EXACTLY, 2) == -1)
    return 0;

  long id = RtnLong(1);
  long cmapId = RtnLong(2);

  return (long)wxcDCSetColourMap(id, cmapId);
}

long clipsDCSetTextForeground()
{
  if (ArgCountCheck("clipsDCSetTextForeground", EXACTLY, 2) == -1)
    return 0;

  long id = RtnLong(1);
  char *colour = RtnLexeme(2);
  return (long)wxcDCSetTextForeground(id, colour);
}

long clipsDCSetTextBackground()
{
  if (ArgCountCheck("clipsDCSetTextBackground", EXACTLY, 2) == -1)
    return 0;

  long id = RtnLong(1);
  char *colour = RtnLexeme(2);
  return (long)wxcDCSetTextBackground(id, colour);
}

long clipsDCSetBackgroundMode()
{
  if (ArgCountCheck("clipsDCSetBackgroundMode", EXACTLY, 2) == -1)
    return 0;

  long id = RtnLong(1);
  char *mode = RtnLexeme(2);
  return (long)wxcDCSetBackgroundMode(id, mode);
}

long clipsDCSetUserScale()
{
  if (ArgCountCheck("clipsDCSetUserScale", EXACTLY, 3) == -1)
    return 0;

  long id = RtnLong(1);

  return (long)wxcDCSetUserScale(id, RtnDouble(2), RtnDouble(3));
}

long clipsDCSetDeviceOrigin()
{
  if (ArgCountCheck("clipsDCSetDeviceOrigin", EXACTLY, 3) == -1)
    return 0;

  long id = RtnLong(1);

  return (long)wxcDCSetDeviceOrigin(id, RtnDouble(2), RtnDouble(3));
}

long clipsDCSetLogicalOrigin()
{
  if (ArgCountCheck("clipsDCSetLogicalOrigin", EXACTLY, 3) == -1)
    return 0;

  long id = RtnLong(1);

  return (long)wxcDCSetLogicalOrigin(id, RtnDouble(2), RtnDouble(3));
}

long clipsDCSetLogicalFunction()
{
  if (ArgCountCheck("clipsDCSetLogicalFunction", EXACTLY, 2) == -1)
    return 0;

  long id = RtnLong(1);
  char *op = RtnLexeme(2);
  return (long)wxcDCSetLogicalFunction(id, op);
}

long clipsDCBlit()
{
  if (ArgCountCheck("clipsDCBlit", AT_LEAST, 8) == -1)
    return 0;

  long id = RtnLong(1);
  double xdest = RtnDouble(2);
  double ydest = RtnDouble(3);
  double width = RtnDouble(4);
  double height = RtnDouble(5);
  long sourceId = RtnLong(6);
  double xsrc = RtnDouble(7);
  double ysrc = RtnDouble(8);
  char *logicalOpString = "wxCOPY";
  if (RtnArgCount() > 8)
    logicalOpString = RtnLexeme(9);
  return (long)wxcDCBlit(id, xdest, ydest, width, height, sourceId, xsrc, ysrc, logicalOpString);
}

long clipsDCDrawLine()
{
  if (ArgCountCheck("clipsDCDrawLine", EXACTLY, 5) == -1)
    return 0;

  long id = RtnLong(1);
  double x1 = RtnDouble(2);
  double y1 = RtnDouble(3);
  double x2 = RtnDouble(4);
  double y2 = RtnDouble(5);
  return (long)wxcDCDrawLine(id, x1, y1, x2, y2);
}

long clipsDCDrawPoint()
{
  if (ArgCountCheck("clipsDCDrawPoint", EXACTLY, 3) == -1)
    return 0;

  long id = RtnLong(1);
  double x1 = RtnDouble(2);
  double y1 = RtnDouble(3);
  return (long)wxcDCDrawPoint(id, x1, y1);
}

long clipsDCDrawRectangle()
{
  if (ArgCountCheck("clipsDCDrawRectangle", EXACTLY, 5) == -1)
    return 0;

  long id = RtnLong(1);
  double x1 = RtnDouble(2);
  double y1 = RtnDouble(3);
  double x2 = RtnDouble(4);
  double y2 = RtnDouble(5);

  return (long)wxcDCDrawRectangle(id, (float)x1, (float)y1, (float)x2, (float)y2);
}

long clipsDCDrawRoundedRectangle()
{
  if (ArgCountCheck("clipsDCDrawRoundedRectangle", AT_LEAST, 5) == -1)
    return 0;

  long id = RtnLong(1);
  double x1 = RtnDouble(2);
  double y1 = RtnDouble(3);
  double x2 = RtnDouble(4);
  double y2 = RtnDouble(5);
  double radius = 20;
  if (RtnArgCount() > 5)
    radius = RtnDouble(6);
  return (long)wxcDCDrawRoundedRectangle(id, x1, y1, x2, y2, radius);
}

long clipsDCDrawEllipse()
{
  if (ArgCountCheck("clipsDCDrawEllipse", EXACTLY, 5) == -1)
    return 0;

  long id = RtnLong(1);
  double x = RtnDouble(2);
  double y = RtnDouble(3);
  double width = RtnDouble(4);
  double height = RtnDouble(5);

  return (long)wxcDCDrawEllipse(id, x, y, width, height);
}

long clipsDCDrawSpline()
{
  DATA_OBJECT arg;

  if (ArgCountCheck("clipsDCDrawSpline", EXACTLY, 2) == -1)
    return 0;

  long id = RtnLong(1);

  wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
  if (dc)
  {
    RtnUnknown(2, &arg);

    if (ClipsGetType(arg) != MULTIFIELD)
    {
      ExtensionError << "Error in dc-draw-spline: 2nd argument must be a multifield of floating-point numbers.\n";
      return 0;
    }

    int start = GetpDOBegin(&arg);
    int end = GetpDOEnd(&arg);

    VOID *multifieldPtr = ClipsGetValue(arg);

    int i;
    wxList pointList;
    for (i = start; i <= end; i += 2)
    {
      double x, y;
      if ((GetMFType(multifieldPtr, i) == FLOAT) || (GetMFType(multifieldPtr, i) == INTEGER))
        x = ValueToDouble(GetMFValue(multifieldPtr, i));
      else
      {
        ExtensionError << "Error in dc-draw-spline: multifield element " << i-start+1 << " must be a number.\n";
        return 0;
      }
      if (((i+1) <= end) && ((GetMFType(multifieldPtr, i+1) == FLOAT) || (GetMFType(multifieldPtr, i+1) == INTEGER)))
        y = ValueToDouble(GetMFValue(multifieldPtr, i+1));
      else
      {
        ExtensionError << "Error in dc-draw-spline: multifield element " << i-start+2 << " must be a number.\n";
        return 0;
      }
#ifndef BE_PARANOID
      pointList.Append(new wxPoint((float)x, (float)y));
#else
      wxPoint *new_pt = new wxPoint ;
      new_pt->x = (float)x ;
      new_pt->y = (float)y ;
      pointList.Append(new_pt) ;
#endif
    }
    dc->DrawSpline(&pointList);
    wxNode *node = pointList.First();
    while (node)
    {
      wxPoint *point = (wxPoint *)node->Data();
      delete point;
      node = node->Next();
    }
    return 1;
  }
  else
  {
    ExtensionError << "Error in dc-draw-spline: could not find dc " << id << ".\n";
    return 0;
  }
}

long clipsDCDrawPolygon()
{
  DATA_OBJECT arg;

  if (ArgCountCheck("clipsDCDrawPolygon", AT_LEAST, 2) == -1)
    return 0;

  double x_offset = 0.0;
  double y_offset = 0.0;
  
  long id = RtnLong(1);
  int no_args = RtnArgCount();
  if (no_args > 2)
    x_offset = RtnDouble(3);
  if (no_args > 3)
    y_offset = RtnDouble(4);

  wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
  if (dc)
  {
    RtnUnknown(2, &arg);

    if (ClipsGetType(arg) != MULTIFIELD)
    {
      ExtensionError << "Error in dc-draw-polygon: 2nd argument must be a multifield of floating-point numbers.\n";
      return 0;
    }

    int start = GetpDOBegin(&arg);
    int end = GetpDOEnd(&arg);

    VOID *multifieldPtr = ClipsGetValue(arg);

    int i;
    wxList pointList;
    for (i = start; i <= end; i += 2)
    {
      double x, y;
      if ((GetMFType(multifieldPtr, i) == FLOAT) || (GetMFType(multifieldPtr, i) == INTEGER))
        x = ValueToDouble(GetMFValue(multifieldPtr, i));
      else
      {
        ExtensionError << "Error in dc-draw-polygon: multifield element " << i-start+1 << " must be a number.\n";
        return 0;
      }
      if (((i+1) <= end) && ((GetMFType(multifieldPtr, i+1) == FLOAT) || (GetMFType(multifieldPtr, i+1) == INTEGER)))
        y = ValueToDouble(GetMFValue(multifieldPtr, i+1));
      else
      {
        ExtensionError << "Error in dc-draw-polygon: multifield element " << i-start+2 << " must be a number.\n";
        return 0;
      }
#ifndef BE_PARANOID
      pointList.Append(new wxPoint((float)x, (float)y));
#else
      wxPoint *new_pt = new wxPoint ;
      new_pt->x = (float)x ;
      new_pt->y = (float)y ;
      pointList.Append(new_pt) ;
#endif
    }
    dc->DrawPolygon(&pointList);
    wxNode *node = pointList.First();
    while (node)
    {
      wxPoint *point = (wxPoint *)node->Data();
      delete point;
      node = node->Next();
    }
    return 1;
  }
  else
  {
    ExtensionError << "Error in dc-draw-polygon: could not find dc " << id << ".\n";
    return 0;
  }
}

long clipsDCDrawLines()
{
  DATA_OBJECT arg;

  if (ArgCountCheck("clipsDCDrawLines", AT_LEAST, 2) == -1)
    return 0;

  double x_offset = 0.0;
  double y_offset = 0.0;
  
  long id = RtnLong(1);
  int no_args = RtnArgCount();
  if (no_args > 2)
    x_offset = RtnDouble(3);
  if (no_args > 3)
    y_offset = RtnDouble(4);

  wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
  if (dc)
  {
    RtnUnknown(2, &arg);

    if (ClipsGetType(arg) != MULTIFIELD)
    {
      ExtensionError << "Error in dc-draw-lines: 2nd argument must be a multifield of floating-point numbers.\n";
      return 0;
    }

    int start = GetpDOBegin(&arg);
    int end = GetpDOEnd(&arg);

    VOID *multifieldPtr = ClipsGetValue(arg);

    int i;
    wxList pointList;
    for (i = start; i <= end; i += 2)
    {
      double x, y;
      if ((GetMFType(multifieldPtr, i) == FLOAT) || (GetMFType(multifieldPtr, i) == INTEGER))
        x = ValueToDouble(GetMFValue(multifieldPtr, i));
      else
      {
        ExtensionError << "Error in dc-draw-lines: multifield element " << i-start+1 << " must be a number.\n";
        return 0;
      }
      if (((i+1) <= end) && ((GetMFType(multifieldPtr, i+1) == FLOAT) || (GetMFType(multifieldPtr, i+1) == INTEGER)))
        y = ValueToDouble(GetMFValue(multifieldPtr, i+1));
      else
      {
        ExtensionError << "Error in dc-draw-lines: multifield element " << i-start+2 << " must be a number.\n";
        return 0;
      }
#ifndef BE_PARANOID
      pointList.Append(new wxPoint((float)x, (float)y));
#else
      wxPoint *new_pt = new wxPoint ;
      new_pt->x = (float)x ;
      new_pt->y = (float)y ;
      pointList.Append(new_pt) ;
#endif
    }
    dc->DrawPolygon(&pointList);
    wxNode *node = pointList.First();
    while (node)
    {
      wxPoint *point = (wxPoint *)node->Data();
      delete point;
      node = node->Next();
    }
    return 1;
  }
  else
  {
    ExtensionError << "Error in dc-draw-lines: could not find dc " << id << ".\n";
    return 0;
  }
}

long clipsDCDrawText()
{
  if (ArgCountCheck("clipsDCDrawText", EXACTLY, 4) == -1)
    return 0;

  long id = RtnLong(1);
  char *text = RtnLexeme(2);
  double x1 = RtnDouble(3);
  double y1 = RtnDouble(4);

  return (long)wxcDCDrawText(id, text, x1, y1);
}

long clipsDCSetClippingRegion()
{
  if (ArgCountCheck("clipsDCSetClippingRegion", EXACTLY, 5) == -1)
    return 0;

  long id = RtnLong(1);
  double x1 = RtnDouble(2);
  double y1 = RtnDouble(3);
  double x2 = RtnDouble(4);
  double y2 = RtnDouble(5);

  return (long)wxcDCSetClippingRegion(id, x1, y1, x2, y2);
}

long clipsDCDestroyClippingRegion()
{
  if (ArgCountCheck("clipsDCDestroyClippingRegion", EXACTLY, 1) == -1)
    return 0;

  long id = RtnLong(1);
  return (long)wxcDCDestroyClippingRegion(id);
}

long clipsDCClear()
{
  if (ArgCountCheck("clipsDCClear", EXACTLY, 1) == -1)
    return 0;

  long id = RtnLong(1);
  return (long)wxcDCClear(id);
}

long clipsDCStartDoc()
{
  if (ArgCountCheck("clipsDCStartDoc", AT_LEAST, 1) == -1)
    return 0;

  long id = RtnLong(1);
  char *s = NULL;

  if (RtnArgCount() > 1)
    s = RtnLexeme(2);

  return (long)wxcDCStartDoc(id, s);
}

long clipsDCEndDoc()
{
  if (ArgCountCheck("clipsDCEndDoc", EXACTLY, 1) == -1)
    return 0;

  long id = RtnLong(1);

  return (long)wxcDCEndDoc(id);
}

long clipsDCStartPage()
{
  if (ArgCountCheck("clipsDCStartPage", EXACTLY, 1) == -1)
    return 0;

  long id = RtnLong(1);

  return (long)wxcDCStartPage(id);
}

long clipsDCEndPage()
{
  if (ArgCountCheck("clipsDCEndPage", EXACTLY, 1) == -1)
    return 0;

  long id = RtnLong(1);

  return (long)wxcDCEndPage(id);
}

long clipsDCDelete()
{
  if (ArgCountCheck("clipsDCDelete", EXACTLY, 1) == -1)
    return 0;

  return (long)wxcDCDelete(RtnLong(1));
}

long clipsPostScriptDCCreate()
{
  if (ArgCountCheck("clipsPostScriptDCCreate", AT_LEAST, 0) == -1)
    return 0;

  char *filename = NULL;
  long interactive = 1;
  long windowId = 0;

  if (RtnArgCount() > 0)
    filename = RtnLexeme(1);
  if (RtnArgCount() > 1)
    interactive = RtnLong(2);
  if (RtnArgCount() > 2)
    windowId = RtnLong(3);

  if (filename && strlen(filename) == 0)
    filename = NULL;
  
  return wxcPostScriptDCCreate(filename, (Bool)interactive, windowId);
}

long clipsMemoryDCCreate()
{
  if (ArgCountCheck("clipsMemoryDCCreate", EXACTLY, 0) == -1)
    return 0;

  return wxcMemoryDCCreate();
}

long clipsMemoryDCSelectObject()
{
  if (ArgCountCheck("clipsMemoryDCSelectObject", EXACTLY, 2) == -1)
    return 0;

  return (long)wxcMemoryDCSelectObject(RtnLong(1), RtnLong(2));
}

#ifdef wx_msw
long clipsPrinterDCCreate()
{
  if (ArgCountCheck("clipsPrinterDCCreate", AT_LEAST, 0) == -1)
    return 0;

  char *driver = NULL;
  char *device = NULL;
  char *filename = NULL;
  long interactive = 1;

  if (RtnArgCount() > 0)
    driver = RtnLexeme(1);
  if (RtnArgCount() > 1)
    device = RtnLexeme(2);
  if (RtnArgCount() > 2)
    filename = RtnLexeme(3);
  if (RtnArgCount() > 3)
    interactive = RtnLong(4);

  if (driver && strlen(driver) == 0)
    driver = NULL;
  if (device && strlen(device) == 0)
    device = NULL;
  if (filename && strlen(filename) == 0)
    filename = NULL;
  
  return wxcPrinterDCCreate(driver, device, filename, (Bool)interactive);
}

long clipsMetaFileDCCreate()
{
  if (ArgCountCheck("clipsMetaFileDCCreate", AT_LEAST, 0) == -1)
    return 0;

  char *filename = NULL;

  if (RtnArgCount() > 0)
    filename = RtnLexeme(1);

  if (filename && strlen(filename) == 0)
    filename = NULL;
    
  return wxcMetaFileDCCreate(filename);
}

long clipsMetaFileDCClose()
{
  if (ArgCountCheck("clipsMetaFileDCClose", EXACTLY, 1) == -1)
    return 0;

  return (long)wxcMetaFileDCClose(RtnLong(1));
}

long clipsMetaFileDelete()
{
  if (ArgCountCheck("clipsMetaFileDelete", EXACTLY, 1) == -1)
    return 0;

  return (long)wxcMetaFileDelete(RtnLong(1));
}

long clipsMetaFileSetClipboard ()
{
  if (ArgCountCheck("clipsMetaFileSetClipboard", AT_LEAST, 1) == -1)
    return 0;

  double width = 0.0;
  double height = 0.0;
  long id = RtnLong(1);
  if (RtnArgCount() > 1)
    width = RtnDouble(2);
  if (RtnArgCount() > 2)
    height = RtnDouble(3);
  return (long)wxcMetaFileSetClipboard(id, width, height);
}

long clipsMakeMetaFilePlaceable()
{
  if (ArgCountCheck("clipsMakeMetaFilePlaceable", AT_LEAST, 5) == -1)
    return 0;

  char *filename = RtnLexeme(1);
  long minX = RtnLong(2);
  long minY = RtnLong(3);
  long maxX = RtnLong(4);
  long maxY = RtnLong(5);
  double scale = 0.0;
  if (RtnArgCount() > 5)
    scale = RtnDouble(6);

  return (long)wxcMakeMetaFilePlaceable(filename, (int)minX, (int)minY, (int)maxX, (int)maxY, scale);
}

#endif // wx_msw

#endif

/*************************************************************/
/* UserFunctions:  The function which informs CLIPS of any   */
/*   user defined functions.  In the default case, there are */
/*   no user defined functions.  To define functions, either */
/*   this function must be replaced by a function with the   */
/*   same name within this file, or this function can be     */
/*   deleted from this file and included in another file.    */
/*   User defined functions may be included in this file or  */
/*   other files.                                            */
/*   Example of redefined UserFunctions:                     */
/*     UserFunctions()                                       */
/*       {                                                   */
/*        DefineFunction("fun1",'i',fun1,"fun1");            */
/*        DefineFunction("other",'f',other,"other");         */
/*       }                                                   */
/*************************************************************/

#ifdef USE_CLIPS
void wxDCUserFunctions()
{  
  DefineFunction("dc-blit", 'l', PTIF clipsDCBlit, "clipsDCBlit");
  DefineFunction("dc-ok", 'l', PTIF clipsDCOk, "clipsDCOk");
  DefineFunction("dc-clear", 'l', PTIF clipsDCClear, "clipsDCClear");
  DefineFunction("dc-destroy-clipping-region", 'l', PTIF clipsDCDestroyClippingRegion, "clipsDCDestroyClippingRegion");
  DefineFunction("dc-draw-ellipse", 'l', PTIF clipsDCDrawEllipse, "clipsDCDrawEllipse");
  DefineFunction("dc-draw-line", 'l', PTIF clipsDCDrawLine, "clipsDCDrawLine");
  DefineFunction("dc-draw-lines", 'l', PTIF clipsDCDrawLines, "clipsDCDrawLines");
  DefineFunction("dc-draw-point", 'l', PTIF clipsDCDrawPoint, "clipsDCDrawPoint");
  DefineFunction("dc-draw-polygon", 'l', PTIF clipsDCDrawPolygon, "clipsDCDrawPolygon");
  DefineFunction("dc-draw-spline", 'l', PTIF clipsDCDrawSpline, "clipsDCDrawSpline");
  DefineFunction("dc-draw-rectangle", 'l', PTIF clipsDCDrawRectangle, "clipsDCDrawRectangle");
  DefineFunction("dc-draw-rounded-rectangle", 'l', PTIF clipsDCDrawRoundedRectangle, "clipsDCDrawRoundedRectangle");
  DefineFunction("dc-draw-text", 'l', PTIF clipsDCDrawText, "clipsDCDrawText");
  DefineFunction("dc-set-clipping-region", 'l', PTIF clipsDCSetClippingRegion, "clipsDCSetClippingRegion");
  DefineFunction("dc-set-pen", 'l', PTIF clipsDCSetPen, "clipsDCSetPen");
  DefineFunction("dc-set-logical-function", 'l', PTIF clipsDCSetLogicalFunction, "clipsDCSetLogicalFunction");
  DefineFunction("dc-set-text-foreground", 'l', PTIF clipsDCSetTextForeground, "clipsDCSetTextForeground");
  DefineFunction("dc-set-text-background", 'l', PTIF clipsDCSetTextBackground, "clipsDCSetTextBackground");
  DefineFunction("dc-set-background", 'l', PTIF clipsDCSetBackground, "clipsDCSetBackground");
  DefineFunction("dc-set-background-mode", 'l', PTIF clipsDCSetBackgroundMode, "clipsDCSetBackgroundMode");
  DefineFunction("dc-set-brush", 'l', PTIF clipsDCSetBrush, "clipsDCSetBrush");
  DefineFunction("dc-set-font", 'l', PTIF clipsDCSetFont, "clipsDCSetFont");
  DefineFunction("dc-set-colourmap", 'l', PTIF clipsDCSetColourMap, "clipsDCSetColourMap");
  DefineFunction("dc-start-doc", 'l', PTIF clipsDCStartDoc, "clipsDCStartDoc");
  DefineFunction("dc-end-doc", 'l', PTIF clipsDCEndDoc, "clipsDCEndDoc");
  DefineFunction("dc-start-page", 'l', PTIF clipsDCStartPage, "clipsDCStartPage");
  DefineFunction("dc-end-page", 'l', PTIF clipsDCEndPage, "clipsDCEndPage");
  DefineFunction("dc-get-width", 'd', PTIF clipsDCGetWidth, "clipsDCGetWidth");
  DefineFunction("dc-get-height", 'd', PTIF clipsDCGetHeight, "clipsDCGetHeight");
  DefineFunction("dc-get-text-extent-height", 'd', PTIF clipsDCGetTextExtentHeight, "clipsDCGetTextExtentHeight");
  DefineFunction("dc-get-text-extent-width", 'd', PTIF clipsDCGetTextExtentWidth, "clipsDCGetTextExtentWidth");
  DefineFunction("dc-get-min-x", 'd', PTIF clipsDCGetMinX, "clipsDCGetMinX");
  DefineFunction("dc-get-min-y", 'd', PTIF clipsDCGetMinY, "clipsDCGetMinY");
  DefineFunction("dc-get-max-x", 'd', PTIF clipsDCGetMaxX, "clipsDCGetMaxX");
  DefineFunction("dc-get-max-y", 'd', PTIF clipsDCGetMaxY, "clipsDCGetMaxY");
  DefineFunction("dc-set-user-scale", 'l', PTIF clipsDCSetUserScale, "clipsDCSetUserScale");
  DefineFunction("dc-set-logical-origin", 'l', PTIF clipsDCSetLogicalOrigin, "clipsDCSetLogicalOrigin");
  DefineFunction("dc-set-device-origin", 'l', PTIF clipsDCSetDeviceOrigin, "clipsDCSetDeviceOrigin");
  DefineFunction("dc-delete", 'l', PTIF clipsDCDelete, "clipsDCDelete");
  DefineFunction("dc-begin-drawing", 'l', PTIF clipsDCBeginDrawing, "clipsDCBeginDrawing");
  DefineFunction("dc-end-drawing", 'l', PTIF clipsDCEndDrawing, "clipsDCEndDrawing");

  DefineFunction("memory-dc-create", 'l', PTIF clipsMemoryDCCreate, "clipsMemoryDCCreate");
  DefineFunction("memory-dc-select-object", 'l', PTIF clipsMemoryDCSelectObject, "clipsMemoryDCSelectObject");

  DefineFunction("postscript-dc-create", 'l', PTIF clipsPostScriptDCCreate, "clipsPostScriptDCCreate");

#ifdef wx_msw
  DefineFunction("printer-dc-create", 'l', PTIF clipsPrinterDCCreate, "clipsPrinterDCCreate");
  DefineFunction("metafile-dc-create", 'l', PTIF clipsMetaFileDCCreate, "clipsMetaFileDCCreate");
  DefineFunction("metafile-dc-close", 'l', PTIF clipsMetaFileDCClose, "clipsMetaFileDCClose");

  DefineFunction("metafile-delete", 'l', PTIF clipsMetaFileDelete, "clipsMetaFileDelete");
  DefineFunction("metafile-set-clipboard", 'l', PTIF clipsMetaFileSetClipboard, "clipsMetaFileSetClipboard");

  DefineFunction("make-metafile-placeable", 'l', PTIF clipsMakeMetaFilePlaceable, "clipsMakeMetaFilePlaceable");
#endif
}
#endif

