/*
 * wxcgrid.cc
 * wxWindows command processing: grid control
 *
 */

// For compilers that support precompilation, includes "wx.h".
#include "wx_prec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx.h"
#endif

#include "wx_hash.h"
#include "wx_types.h"

#include <iostream.h>
#include <ctype.h>
#include <stdlib.h>

#include "wx_cmds.h"
#include "wxcmdsp.h"

#include "wxgrid.h"

#ifdef USE_CLIPS
#ifndef __WATCOMC__
#include "clips.h"
#endif
#endif

void wxClipsGridOnCellLeftClick(Bool *handled, WXID id, int row, int col, int x, int y, Bool controlDown, Bool shiftDown)
{
  wxClipsFunction func = wxClipsFindCallback(id, "OnCellLeftClick");
  if (func)
  {
    *handled = TRUE;
        
#ifdef USE_CLIPS
    char buf[100];
    sprintf(buf, "(%s %ld %d %d %d %d %d %d)", func, id, row, col, x, y, controlDown, shiftDown);

    wxRouteNoEcho(buf);
#else
    (* (void (*)(long, long, long, long, long, long, long)) func) (id, (long)row, (long)col, (long)x, (long)y,
        (long)controlDown, (long)shiftDown);
#endif
  }
}

void wxClipsGridOnCellRightClick(Bool *handled, WXID id, int row, int col, int x, int y, Bool controlDown, Bool shiftDown)
{
  wxClipsFunction func = wxClipsFindCallback(id, "OnCellRightClick");
  if (func)
  {
    *handled = TRUE;
        
#ifdef USE_CLIPS
    char buf[100];
    sprintf(buf, "(%s %ld %d %d %d %d %d %d)", func, id, row, col, x, y, controlDown, shiftDown);

    wxRouteNoEcho(buf);
#else
    (* (void (*)(long, long, long, long, long, long, long)) func) (id, (long)row, (long)col, (long)x, (long)y,
        (long)controlDown, (long)shiftDown);
#endif
  }
}

void wxClipsGridOnCellChange(Bool *handled, WXID id, int row, int col)
{
  wxClipsFunction func = wxClipsFindCallback(id, "OnCellChange");
  if (func)
  {
    *handled = TRUE;
        
#ifdef USE_CLIPS
    char buf[100];
    sprintf(buf, "(%s %ld %d %d)", func, id, row, col);

    wxRouteNoEcho(buf);
#else
    (* (void (*)(long, long, long)) func) (id, (long)row, (long)col);
#endif
  }
}


void wxClipsGridOnChangeLabels(Bool *handled, WXID id)
{
  wxClipsFunction func = wxClipsFindCallback(id, "OnChangeLabels");
  if (func)
  {
    *handled = TRUE;
        
#ifdef USE_CLIPS
    char buf[100];
    sprintf(buf, "(%s %ld)", func, id);

    wxRouteNoEcho(buf);
#else
    (* (void (*)(long)) func) (id);
#endif
  }
}

void wxClipsGridOnChangeSelectionLabel(Bool *handled, WXID id)
{
  wxClipsFunction func = wxClipsFindCallback(id, "OnChangeSelectionLabel");
  if (func)
  {
    *handled = TRUE;
        
#ifdef USE_CLIPS
    char buf[100];
    sprintf(buf, "(%s %ld)", func, id);

    wxRouteNoEcho(buf);
#else
    (* (void (*)(long)) func) (id);
#endif
  }
}

long clipsGridCreate()
{
  if (ArgCountCheck("clipsGridCreate", AT_LEAST, 1) == -1)
    return -1;

  long parentId = RtnLong(1);
  int x = -1;
  int y = -1;
  int width = -1;
  int height = -1;
  char *style = 0;
  char *name = "grid";
  if (RtnArgCount() > 1)
    x = (int)RtnLong(2);
  if (RtnArgCount() > 2)
    y = (int)RtnLong(3);
  if (RtnArgCount() > 3)
    width = (int)RtnLong(4);
  if (RtnArgCount() > 4)
    height = (int)RtnLong(5);
  if (RtnArgCount() > 5)
    style = RtnLexeme(6);
  if (RtnArgCount() > 6)
    name = RtnLexeme(7);
  return (long)wxcGridCreate(parentId, x, y, width, height, style, name);
}

long clipsGridCreateGrid()
{
  if (ArgCountCheck("clipsGridCreateGrid", AT_LEAST, 3) == -1)
    return -1;

  long id = RtnLong(1);
  int defaultWidth = wxGRID_DEFAULT_CELL_WIDTH;
  int defaultHeight = wxGRID_DEFAULT_CELL_HEIGHT;
  if (RtnArgCount() > 3)
    defaultWidth = (int)RtnLong(4);
  if (RtnArgCount() > 4)
    defaultHeight = (int)RtnLong(5);
  return (long)wxcGridCreateGrid(id, (int)RtnLong(2), (int)RtnLong(3), defaultWidth, defaultHeight);
}

void clipsGridClearGrid()
{
  if (ArgCountCheck("clipsGridClearGrid", EXACTLY, 1) == -1)
    return;

  long id = RtnLong(1);
  wxcGridClearGrid(id);
}

long clipsGridInsertCols()
{
  if (ArgCountCheck("clipsGridInsertCols", AT_LEAST, 3) == -1)
    return 0;

  long id = RtnLong(1);
  Bool updateLabels = TRUE;
  if (RtnArgCount() > 3)
    updateLabels = (Bool)RtnLong(4);
  return (long)wxcGridInsertCols(id, (int)RtnLong(2), (int)RtnLong(3), updateLabels);
}

long clipsGridInsertRows()
{
  if (ArgCountCheck("clipsGridInsertRows", AT_LEAST, 3) == -1)
    return 0;

  long id = RtnLong(1);
  Bool updateLabels = TRUE;
  if (RtnArgCount() > 3)
    updateLabels = (Bool)RtnLong(4);
  return (long)wxcGridInsertRows(id, (int)RtnLong(2), (int)RtnLong(3), updateLabels);
}

long clipsGridAppendCols()
{
  if (ArgCountCheck("clipsGridAppendCols", AT_LEAST, 2) == -1)
    return 0;

  long id = RtnLong(1);
  Bool updateLabels = TRUE;
  if (RtnArgCount() > 2)
    updateLabels = (Bool)RtnLong(3);
  return (long)wxcGridAppendCols(id, (int)RtnLong(2), updateLabels);
}

long clipsGridAppendRows()
{
  if (ArgCountCheck("clipsGridAppendRows", AT_LEAST, 2) == -1)
    return 0;

  long id = RtnLong(1);
  Bool updateLabels = TRUE;
  if (RtnArgCount() > 2)
    updateLabels = (Bool)RtnLong(3);
  return (long)wxcGridAppendRows(id, (int)RtnLong(2), updateLabels);
}

long clipsGridDeleteCols()
{
  if (ArgCountCheck("clipsGridDeleteCols", AT_LEAST, 3) == -1)
    return 0;

  long id = RtnLong(1);
  Bool updateLabels = TRUE;
  if (RtnArgCount() > 3)
    updateLabels = (Bool)RtnLong(4);
  return (long)wxcGridDeleteCols(id, (int)RtnLong(2), (int)RtnLong(3), updateLabels);
}

long clipsGridDeleteRows()
{
  if (ArgCountCheck("clipsGridDeleteRows", AT_LEAST, 3) == -1)
    return 0;

  long id = RtnLong(1);
  Bool updateLabels = TRUE;
  if (RtnArgCount() > 3)
    updateLabels = (Bool)RtnLong(4);
  return (long)wxcGridDeleteRows(id, (int)RtnLong(2), (int)RtnLong(3), updateLabels);
}

void clipsGridSetCellValue()
{
  if (ArgCountCheck("clipsGridSetCellValue", EXACTLY, 4) == -1)
    return;

  long id = RtnLong(1);
  char *val = RtnLexeme(2);
  if (!val)
    return;
    
  int row = (int)RtnLong(3);
  int col = (int)RtnLong(4);
  wxcGridSetCellValue(id, val, row, col);
}

void clipsGridSetCellBitmap()
{
  if (ArgCountCheck("clipsGridSetCellBitmap", EXACTLY, 4) == -1)
    return;

  long id = RtnLong(1);
  long bitmapId = RtnLong(2);
    
  int row = (int)RtnLong(3);
  int col = (int)RtnLong(4);
  wxcGridSetCellBitmap(id, bitmapId, row, col);
}

char *clipsGridGetCellValue()
{
  if (ArgCountCheck("clipsGridGetCellValue", EXACTLY, 3) == -1)
    return (char *)AddSymbol("");

  long id = RtnLong(1);
  int row = (int)RtnLong(2);
  int col = (int)RtnLong(3);
  char *s = wxcGridGetCellValue(id, row, col);
  if (s)
    return (char *)AddSymbol(s);
  else
    return (char *)AddSymbol("");
}

long clipsGridGetCellBitmap()
{
  if (ArgCountCheck("clipsGridGetCellBitmap", EXACTLY, 3) == -1)
    return -1;

  long id = RtnLong(1);
  int row = (int)RtnLong(2);
  int col = (int)RtnLong(3);
  return wxcGridGetCellBitmap(id, row, col);
}

void clipsGridSetCellAlignment()
{
  if (ArgCountCheck("clipsGridSetCellAlignment", EXACTLY, 4) == -1)
    return;

  long id = RtnLong(1);
  char *align = RtnLexeme(2);
  if (!align)
    return;
  int row = (int)RtnLong(3);
  int col = (int)RtnLong(4);
  wxcGridSetCellAlignment(id, align, row, col);
}

char *clipsGridGetCellAlignment()
{
  if (ArgCountCheck("clipsGridGetCellAlignment", EXACTLY, 3) == -1)
    return (char *)AddSymbol("");

  long id = RtnLong(1);
  int row = (int)RtnLong(2);
  int col = (int)RtnLong(3);
  char *align = wxcGridGetCellAlignmentString(id, row, col);
  if (align)
    return (char *)AddSymbol(align);
  else
    return (char *)AddSymbol("");
}

void clipsGridSetCellTextColour()
{
  if (ArgCountCheck("clipsGridSetCellTextColour", AT_LEAST, 2) == -1)
    return;

  long id = RtnLong(1);
  long colour = RtnLong(2);
  int row = -1;
  int col = -1;
  if (RtnArgCount() > 2)
    row = (int)RtnLong(3);
  if (RtnArgCount() > 3)
    col = (int)RtnLong(4);
  wxcGridSetCellTextColour(id, (WXCOLOUR)colour, row, col);
}

long clipsGridGetCellTextColour()
{
  if (ArgCountCheck("clipsGridGetCellTextColour", AT_LEAST, 1) == -1)
    return 0;

  long id = RtnLong(1);
  int row = -1;
  int col = -1;
  if (RtnArgCount() > 1)
    row = (int)RtnLong(2);
  if (RtnArgCount() > 2)
    col = (int)RtnLong(3);
  return (long)wxcGridGetCellTextColour(id, row, col);
}

void clipsGridSetCellBackgroundColour()
{
  if (ArgCountCheck("clipsGridSetCellBackgroundColour", AT_LEAST, 2) == -1)
    return;

  long id = RtnLong(1);
  long colour = RtnLong(2);
  int row = -1;
  int col = -1;
  if (RtnArgCount() > 2)
    row = (int)RtnLong(3);
  if (RtnArgCount() > 3)
    col = (int)RtnLong(4);
  wxcGridSetCellBackgroundColour(id, (WXCOLOUR)colour, row, col);
}

long clipsGridGetCellBackgroundColour()
{
  if (ArgCountCheck("clipsGridGetCellBackgroundColour", AT_LEAST, 1) == -1)
    return 0;

  long id = RtnLong(1);
  int row = -1;
  int col = -1;
  if (RtnArgCount() > 1)
    row = (int)RtnLong(2);
  if (RtnArgCount() > 2)
    col = (int)RtnLong(3);
  return (long)wxcGridGetCellBackgroundColour(id, row, col);
}

void clipsGridSetCellTextFont()
{
  if (ArgCountCheck("clipsGridSetCellTextFont", AT_LEAST, 2) == -1)
    return;

  long id = RtnLong(1);
  long font = RtnLong(2);
  int row = -1;
  int col = -1;
  if (RtnArgCount() > 2)
    row = (int)RtnLong(3);
  if (RtnArgCount() > 3)
    col = (int)RtnLong(4);
  wxcGridSetCellTextFont(id, font, row, col);
}

void clipsGridSetColumnWidth()
{
  if (ArgCountCheck("clipsGridSetColumnWidth", EXACTLY, 3) == -1)
    return;

  wxcGridSetColumnWidth(RtnLong(1), (int)RtnLong(2), (int)RtnLong(3));
}

long clipsGridGetColumnWidth()
{
  if (ArgCountCheck("clipsGridGetColumnWidth", EXACTLY, 2) == -1)
    return 0;

  return (long)wxcGridGetColumnWidth(RtnLong(1), (int)RtnLong(2));
}

void clipsGridSetRowHeight()
{
  if (ArgCountCheck("clipsGridSetRowHeight", EXACTLY, 3) == -1)
    return;

  wxcGridSetRowHeight(RtnLong(1), (int)RtnLong(2), (int)RtnLong(3));
}

long clipsGridGetRowHeight()
{
  if (ArgCountCheck("clipsGridGetRowHeight", EXACTLY, 2) == -1)
    return 0;

  return (long)wxcGridGetRowHeight(RtnLong(1), (int)RtnLong(2));
}

void clipsGridSetLabelSize()
{
  if (ArgCountCheck("clipsGridSetLabelSize", EXACTLY, 3) == -1)
    return;

  wxcGridSetLabelSize(RtnLong(1), RtnLexeme(2), (int)RtnLong(3));
}

long clipsGridGetLabelSize()
{
  if (ArgCountCheck("clipsGridGetLabelSize", EXACTLY, 2) == -1)
    return 0;

  return (long)wxcGridGetLabelSize(RtnLong(1), RtnLexeme(2));
}

void clipsGridSetLabelAlignment()
{
  if (ArgCountCheck("clipsGridSetLabelAlignment", EXACTLY, 3) == -1)
    return;

  wxcGridSetLabelAlignment(RtnLong(1), RtnLexeme(2), RtnLexeme(3));
}

char *clipsGridGetLabelAlignment()
{
  if (ArgCountCheck("clipsGridGetLabelAlignment", EXACTLY, 2) == -1)
    return (char *)AddSymbol("");

  char *alignment = wxcGridGetLabelAlignment(RtnLong(1), RtnLexeme(2));
  if (alignment)
    return (char *)AddSymbol(alignment);
  else
    return (char *)AddSymbol("");
}

void clipsGridSetLabelValue()
{
  if (ArgCountCheck("clipsGridSetLabelValue", EXACTLY, 4) == -1)
    return;

  wxcGridSetLabelValue(RtnLong(1), RtnLexeme(2), RtnLexeme(3), (int)RtnLong(4));
}

char *clipsGridGetLabelValue()
{
  if (ArgCountCheck("clipsGridGetLabelValue", EXACTLY, 3) == -1)
    return (char *)AddSymbol("");

  char *value = wxcGridGetLabelValue(RtnLong(1), RtnLexeme(2), (int)RtnLong(3));
  if (value)
    return (char *)AddSymbol(value);
  else
    return (char *)AddSymbol("");
}

void clipsGridSetLabelTextColour()
{
  if (ArgCountCheck("clipsGridSetLabelTextColour", EXACTLY, 2) == -1)
    return;

  wxcGridSetLabelTextColour(RtnLong(1), (WXCOLOUR)RtnLong(2));
}

long clipsGridGetLabelTextColour()
{
  if (ArgCountCheck("clipsGridGetLabelTextColour", EXACTLY, 1) == -1)
    return 0;

  return (long)wxcGridGetLabelTextColour(RtnLong(1));
}


void clipsGridSetLabelBackgroundColour()
{
  if (ArgCountCheck("clipsGridSetLabelBakckgroundColour", EXACTLY, 2) == -1)
    return;

  wxcGridSetLabelBackgroundColour(RtnLong(1), (WXCOLOUR)RtnLong(2));
}

long clipsGridGetLabelBackgroundColour()
{
  if (ArgCountCheck("clipsGridGetLabelBackgroundColour", EXACTLY, 1) == -1)
    return 0;

  return (long)wxcGridGetLabelBackgroundColour(RtnLong(1));
}

void clipsGridSetLabelTextFont()
{
  if (ArgCountCheck("clipsGridSetLabelTextFont", EXACTLY, 2) == -1)
    return;

  wxcGridSetLabelTextFont(RtnLong(1), RtnLong(2));
}

long clipsGridGetCursorRow()
{
  if (ArgCountCheck("clipsGridGetCursorRow", EXACTLY, 1) == -1)
    return 0;

  return (long)wxcGridGetCursorRow(RtnLong(1));
}

long clipsGridGetCursorColumn()
{
  if (ArgCountCheck("clipsGridGetCursorColumn", EXACTLY, 1) == -1)
    return 0;

  return (long)wxcGridGetCursorColumn(RtnLong(1));
}

void clipsGridSetGridCursor()
{
  if (ArgCountCheck("clipsGridSetGridCursor", EXACTLY, 3) == -1)
    return;

  wxcGridSetGridCursor(RtnLong(1), (int)RtnLong(2), (int)RtnLong(3));
}

long clipsGridGetRows()
{
  if (ArgCountCheck("clipsGridGetRows", EXACTLY, 1) == -1)
    return 0;

  return (long)wxcGridGetRows(RtnLong(1));
}

long clipsGridGetCols()
{
  if (ArgCountCheck("clipsGridGetCols", EXACTLY, 1) == -1)
    return 0;

  return (long)wxcGridGetCols(RtnLong(1));
}

long clipsGridGetScrollPosX()
{
  if (ArgCountCheck("clipsGridGetScrollPosX", EXACTLY, 1) == -1)
    return 0;

  return (long)wxcGridGetScrollPosX(RtnLong(1));
}

long clipsGridGetScrollPosY()
{
  if (ArgCountCheck("clipsGridGetScrollPosY", EXACTLY, 1) == -1)
    return 0;

  return (long)wxcGridGetScrollPosY(RtnLong(1));
}

void clipsGridSetEditable()
{
  if (ArgCountCheck("clipsGridSetEditable", EXACTLY, 2) == -1)
    return;

  wxcGridSetEditable(RtnLong(1), (Bool)RtnLong(2));
}

long clipsGridGetEditable()
{
  if (ArgCountCheck("clipsGridGetEditable", EXACTLY, 1) == -1)
    return 0;

  return (long)wxcGridGetEditable(RtnLong(1));
}

void clipsGridSetDividerPen()
{
  if (ArgCountCheck("clipsGridSetDividerPen", EXACTLY, 2) == -1)
    return;

  wxcGridSetDividerPen(RtnLong(1), RtnLong(2));
}

void clipsGridAdjustScrollbars()
{
  if (ArgCountCheck("clipsGridAdjustScrollbars", EXACTLY, 1) == -1)
    return;

  wxcGridAdjustScrollbars(RtnLong(1));
}

void clipsGridUpdateDimensions()
{
  if (ArgCountCheck("clipsGridUpdateDimensions", EXACTLY, 1) == -1)
    return;

  wxcGridUpdateDimensions(RtnLong(1));
}

void clipsGridOnActivate()
{
  if (ArgCountCheck("clipsGridOnActivate", EXACTLY, 2) == -1)
    return;

  wxcGridOnActivate(RtnLong(1), (Bool)RtnLong(2));
}

void clipsGridOnPaint()
{
  if (ArgCountCheck("clipsGridOnPaint", EXACTLY, 1) == -1)
    return;

  wxcGridOnPaint(RtnLong(1));
}

void clipsGridOnSize()
{
  if (ArgCountCheck("clipsGridOnSize", EXACTLY, 3) == -1)
    return;

  wxcGridOnSize(RtnLong(1), (int)RtnLong(2), (int)RtnLong(3));
}

long clipsGridGetTextItem()
{
  if (ArgCountCheck("clipsGridGetTextItem", EXACTLY, 1) == -1)
    return -1;

  return wxcGridGetTextItem(RtnLong(1));
}


/*************************************************************/
/* UserFunctions:  The function which informs CLIPS of any   */
/*   user defined functions.  In the default case, there are */
/*   no user defined functions.  To define functions, either */
/*   this function must be replaced by a function with the   */
/*   same name within this file, or this function can be     */
/*   deleted from this file and included in another file.    */
/*   User defined functions may be included in this file or  */
/*   other files.                                            */
/*   Example of redefined UserFunctions:                     */
/*     UserFunctions()                                       */
/*       {                                                   */
/*        DefineFunction("fun1",'i',fun1,"fun1");            */
/*        DefineFunction("other",'f',other,"other");         */
/*       }                                                   */
/*************************************************************/

#ifdef USE_CLIPS
void wxGridUserFunctions()
{  
  DefineFunction("grid-create", 'l', PTIF clipsGridCreate, "clipsGridCreate");
  DefineFunction("grid-create-grid", 'l', PTIF clipsGridCreateGrid, "clipsGridCreateGrid");
  DefineFunction("grid-clear-grid", 'v', PTIF clipsGridClearGrid, "clipsGridClearGrid");
  DefineFunction("grid-insert-cols", 'l', PTIF clipsGridInsertCols, "clipsGridInsertCols");
  DefineFunction("grid-insert-rows", 'l', PTIF clipsGridInsertRows, "clipsGridInsertRows");
  DefineFunction("grid-append-cols", 'l', PTIF clipsGridAppendCols, "clipsGridAppendCols");
  DefineFunction("grid-append-rows", 'l', PTIF clipsGridAppendRows, "clipsGridAppendRows");
  DefineFunction("grid-delete-cols", 'l', PTIF clipsGridDeleteCols, "clipsGridDeleteCols");
  DefineFunction("grid-delete-rows", 'l', PTIF clipsGridDeleteRows, "clipsGridDeleteRows");
  DefineFunction("grid-set-cell-value", 'v', PTIF clipsGridSetCellValue, "clipsGridSetCellValue");
  DefineFunction("grid-get-cell-value", 'v', PTIF clipsGridGetCellValue, "clipsGridGetCellValue");
  DefineFunction("grid-set-cell-bitmap", 'v', PTIF clipsGridSetCellBitmap, "clipsGridSetCellBitmap");
  DefineFunction("grid-get-cell-bitmap", 'l', PTIF clipsGridGetCellBitmap, "clipsGridGetCellBitmap");
  DefineFunction("grid-set-cell-alignment", 'v', PTIF clipsGridSetCellAlignment, "clipsGridSetCellAlignment");
  DefineFunction("grid-get-cell-alignment", 's', PTIF clipsGridGetCellAlignment, "clipsGridGetCellAlignment");
  DefineFunction("grid-set-cell-text-colour", 'v', PTIF clipsGridSetCellTextColour, "clipsGridSetCellTextColour");
  DefineFunction("grid-get-cell-text-colour", 'l', PTIF clipsGridGetCellTextColour, "clipsGridGetCellTextColour");
  DefineFunction("grid-set-cell-background-colour", 'v', PTIF clipsGridSetCellBackgroundColour, "clipsGridSetCellBackgroundColour");
  DefineFunction("grid-get-cell-background-colour", 'l', PTIF clipsGridGetCellBackgroundColour, "clipsGridGetCellBackgroundColour");
  DefineFunction("grid-set-cell-text-font", 'v', PTIF clipsGridSetCellTextFont, "clipsGridSetCellTextFont");
  DefineFunction("grid-set-column-width", 'v', PTIF clipsGridSetColumnWidth, "clipsGridSetColumnWidth");
  DefineFunction("grid-get-column-width", 'l', PTIF clipsGridGetColumnWidth, "clipsGridGetColumnWidth");
  DefineFunction("grid-set-row-height", 'v', PTIF clipsGridSetRowHeight, "clipsGridSetRowHeight");
  DefineFunction("grid-get-row-height", 'l', PTIF clipsGridGetRowHeight, "clipsGridGetRowHeight");
  DefineFunction("grid-set-label-size", 'v', PTIF clipsGridSetLabelSize, "clipsGridSetLabelSize");
  DefineFunction("grid-get-label-size", 'l', PTIF clipsGridGetLabelSize, "clipsGridGetLabelSize");
  DefineFunction("grid-set-label-alignment", 'v', PTIF clipsGridSetLabelAlignment, "clipsGridSetLabelAlignment");
  DefineFunction("grid-get-label-alignment", 's', PTIF clipsGridGetLabelAlignment, "clipsGridGetLabelAlignment");
  DefineFunction("grid-set-label-value", 'v', PTIF clipsGridSetLabelValue, "clipsGridSetLabelValue");
  DefineFunction("grid-get-label-value", 's', PTIF clipsGridGetLabelValue, "clipsGridGetLabelValue");
  DefineFunction("grid-set-label-text-colour", 'v', PTIF clipsGridSetLabelTextColour, "clipsGridSetLabelTextColour");
  DefineFunction("grid-get-label-text-colour", 'l', PTIF clipsGridGetLabelTextColour, "clipsGridGetLabelTextColour");
  DefineFunction("grid-set-label-background-colour", 'v', PTIF clipsGridSetLabelBackgroundColour, "clipsGridSetLabelBackgroundColour");
  DefineFunction("grid-get-label-background-colour", 'l', PTIF clipsGridGetLabelBackgroundColour, "clipsGridGetLabelBackgroundColour");
  DefineFunction("grid-set-label-text-font", 'v', PTIF clipsGridSetLabelTextFont, "clipsGridSetLabelTextFont");
  DefineFunction("grid-get-cursor-row", 'l', PTIF clipsGridGetCursorRow, "clipsGridGetCursorRow");
  DefineFunction("grid-get-cursor-column", 'l', PTIF clipsGridGetCursorColumn, "clipsGridGetCursorColumn");
  DefineFunction("grid-set-grid-cursor", 'v', PTIF clipsGridSetGridCursor, "clipsGridSetGridCursor");
  DefineFunction("grid-get-rows", 'l', PTIF clipsGridGetRows, "clipsGridGetRows");
  DefineFunction("grid-get-cols", 'l', PTIF clipsGridGetCols, "clipsGridGetCols");
  DefineFunction("grid-get-scroll-pos-x", 'l', PTIF clipsGridGetScrollPosX, "clipsGridGetScrollPosX");
  DefineFunction("grid-get-scroll-pos-y", 'l', PTIF clipsGridGetScrollPosY, "clipsGridGetScrollPosY");
  DefineFunction("grid-set-editable", 'v', PTIF clipsGridSetEditable, "clipsGridSetEditable");
  DefineFunction("grid-get-editable", 'l', PTIF clipsGridGetEditable, "clipsGridGetEditable");
  DefineFunction("grid-set-divider-pen", 'v', PTIF clipsGridSetDividerPen, "clipsGridSetDividerPen");
  DefineFunction("grid-adjust-scrollbars", 'v', PTIF clipsGridAdjustScrollbars, "clipsGridAdjustScrollbars");
  DefineFunction("grid-update-dimensions", 'v', PTIF clipsGridUpdateDimensions, "clipsGridUpdateDimensions");
  DefineFunction("grid-on-activate", 'v', PTIF clipsGridOnActivate, "clipsGridOnActivate");
  DefineFunction("grid-on-size", 'v', PTIF clipsGridOnSize, "clipsGridOnSize");
  DefineFunction("grid-on-paint", 'v', PTIF clipsGridOnPaint, "clipsGridOnPaint");
  DefineFunction("grid-get-text-item", 'l', PTIF clipsGridGetTextItem, "clipsGridGetTextItem");
}
#endif

