/*
 * wxchtml.cc
 * wxWindows command processing: wxHtml control
 *
 */

// For compilers that support precompilation, includes "wx.h".
#include "wx_prec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx.h"
#endif

#include "wx_hash.h"
#include "wx_types.h"

#include <iostream.h>
#include <ctype.h>
#include <stdlib.h>

#include "wx_cmds.h"
#include "wxcmdsp.h"

#include "exhtml.h"

#ifdef USE_CLIPS
#ifndef __WATCOMC__
#include "clips.h"
#endif
#endif

#if USE_WXHTML

long clipsHtmlCreate()
{
  if (ArgCountCheck("clipsHtmlCreate", AT_LEAST, 1) == -1)
    return -1;

  long parentId = RtnLong(1);
  int x = -1;
  int y = -1;
  int width = -1;
  int height = -1;
  char *style = 0;
  char *name = "grid";
  if (RtnArgCount() > 1)
    x = (int)RtnLong(2);
  if (RtnArgCount() > 2)
    y = (int)RtnLong(3);
  if (RtnArgCount() > 3)
    width = (int)RtnLong(4);
  if (RtnArgCount() > 4)
    height = (int)RtnLong(5);
  if (RtnArgCount() > 5)
    style = RtnLexeme(6);
  if (RtnArgCount() > 6)
    name = RtnLexeme(7);
  return (long)wxcHtmlCreate(parentId, x, y, width, height, style, name);
}

long clipsHtmlOpenURL()
{
  if (ArgCountCheck("clipsHtmlOpenURL", EXACTLY, 2) == -1)
    return 0;

  return (long)wxcHtmlOpenURL(RtnLong(1), RtnLexeme(2));
}

long clipsHtmlOpenFile()
{
  if (ArgCountCheck("clipsHtmlOpenFile", EXACTLY, 2) == -1)
    return 0;

  return (long)wxcHtmlOpenFile(RtnLong(1), RtnLexeme(2));
}

long clipsHtmlSaveFile()
{
  if (ArgCountCheck("clipsHtmlSaveFile", EXACTLY, 2) == -1)
    return 0;

  return (long)wxcHtmlSaveFile(RtnLong(1), RtnLexeme(2));
}

char *clipsHtmlGetCurrentURL()
{
  if (ArgCountCheck("clipsHtmlGetCurrentURL", EXACTLY, 1) == -1)
    return (char *)AddSymbol("");

  char *s = wxcHtmlGetCurrentURL(RtnLong(1));
  if (s)
    return (char *)AddSymbol(s);
  else
    return (char *)AddSymbol("");
}

void clipsHtmlResize()
{
  if (ArgCountCheck("clipsHtmlResize", EXACTLY, 1) == -1)
    return;

  wxcHtmlResize(RtnLong(1));
}

void clipsHtmlBack()
{
  if (ArgCountCheck("clipsHtmlBack", EXACTLY, 1) == -1)
    return;

  wxcHtmlBack(RtnLong(1));
}

void clipsHtmlClearCache()
{
  if (ArgCountCheck("clipsHtmlClearCache", EXACTLY, 1) == -1)
    return;

  wxcHtmlClearCache(RtnLong(1));
}

void clipsHtmlCancel()
{
  if (ArgCountCheck("clipsHtmlCancel", EXACTLY, 1) == -1)
    return;

  wxcHtmlCancel(RtnLong(1));
}

/*
void clipsHtmlOnPaint()
{
  if (ArgCountCheck("clipsHtmlOnPaint", EXACTLY, 1) == -1)
    return;

  wxcHtmlOnPaint(RtnLong(1));
}
*/

void clipsHtmlOnSize()
{
  if (ArgCountCheck("clipsHtmlOnSize", EXACTLY, 3) == -1)
    return;

  wxcHtmlOnSize(RtnLong(1), (int)RtnLong(2), (int)RtnLong(3));
}


/*************************************************************/
/* UserFunctions:  The function which informs CLIPS of any   */
/*   user defined functions.  In the default case, there are */
/*   no user defined functions.  To define functions, either */
/*   this function must be replaced by a function with the   */
/*   same name within this file, or this function can be     */
/*   deleted from this file and included in another file.    */
/*   User defined functions may be included in this file or  */
/*   other files.                                            */
/*   Example of redefined UserFunctions:                     */
/*     UserFunctions()                                       */
/*       {                                                   */
/*        DefineFunction("fun1",'i',fun1,"fun1");            */
/*        DefineFunction("other",'f',other,"other");         */
/*       }                                                   */
/*************************************************************/

#ifdef USE_CLIPS
void wxHtmlUserFunctions()
{  
  DefineFunction("html-create", 'l', PTIF clipsHtmlCreate, "clipsHtmlCreate");
  DefineFunction("html-open-url", 'l', PTIF clipsHtmlOpenURL, "clipsHtmlOpenURL");
  DefineFunction("html-open-file", 'l', PTIF clipsHtmlOpenFile, "clipsHtmlOpenFile");
  DefineFunction("html-save-file", 'l', PTIF clipsHtmlSaveFile, "clipsHtmlSaveFile");
  DefineFunction("html-get-current-url", 's', PTIF clipsHtmlGetCurrentURL, "clipsHtmlGetCurrentURL");
  DefineFunction("html-resize", 'v', PTIF clipsHtmlResize, "clipsHtmlResize");
  DefineFunction("html-back", 'v', PTIF clipsHtmlBack, "clipsHtmlBack");
  DefineFunction("html-clear-cache", 'v', PTIF clipsHtmlClearCache, "clipsHtmlClearCache");
  DefineFunction("html-cancel", 'v', PTIF clipsHtmlCancel, "clipsHtmlCancel");
  DefineFunction("html-on-size", 'v', PTIF clipsHtmlOnSize, "clipsHtmlOnSize");
//  DefineFunction("html-on-paint", 'v', PTIF clipsHtmlOnPaint, "clipsHtmlOnPaint");
}
#endif

#endif
 // USE_WXHTML
