/*
 * wxclips.h
 *
 * CLIPS stuff
 *
 */

#ifndef wxclipsh
#define wxclipsh

// Define a new application
class MyApp: public wxApp
{
  public:
    wxFrame *OnInit(void);
#ifdef wx_msw
    Bool OnCharHook(wxKeyEvent& event);
#endif
};

#ifdef wx_msw
class wxClipsToolBar;
#endif
// Define a new frame
class wxClipsEnvFrame: public wxFrame
{
  public:
    wxPanel *panel;
    wxText *command_line;
    wxTextWindow *text_window;
#ifdef wx_msw
    wxClipsToolBar *toolbar;
#endif
    wxClipsEnvFrame(wxFrame *frame, char *title, int x, int y, int w, int h);
    ~wxClipsEnvFrame(void);
    void OnSize(int w, int h);
    Bool OnClose(void);
    void OnActivate(Bool active);
    void OnMenuCommand(int id);
    char *PromptForRuleName(void);
};

// Callbacks
void EnterCommand(wxButton& but, wxCommandEvent& event);
void GenericOk(wxButton& but, wxCommandEvent& event);

// My I/O Routers

extern "C" int DummyGetc(char *c);
extern "C" int DummyUngetc(int i, char *c);
extern "C" int FindOutput(char *name);
extern "C" int PrintOutput(char *name, char *str);
extern "C" int ExitOutput(int exitcode);
extern "C" int ClipsYield(void);

// File menu
#define WXCLIPS_LOAD			  1
#define WXCLIPS_SAVE			  2
#define WXCLIPS_BLOAD			  3
#define WXCLIPS_BSAVE			  4
#define WXCLIPS_QUIT			  5
#define WXCLIPS_BATCH                     6

// Rules menu
#define WXCLIPS_RESETRUN		  20
#define WXCLIPS_RUN 			  21
#define WXCLIPS_RUN_LIMIT		  22
#define WXCLIPS_RESET			  23
#define WXCLIPS_CLEAR			  24
#define WXCLIPS_WATCHES 		  26
#define WXCLIPS_ADD_BREAKPOINT	  27
#define WXCLIPS_REMOVE_BREAKPOINT 28
#define WXCLIPS_SET_STRATEGY	  29
#define WXCLIPS_ASSERT_FACT 	  30

// Application menu
#define WXCLIPS_RUN_APP           40
#define WXCLIPS_CLEAN_WINDOWS     41

// List menu
#define WXCLIPS_LIST_AGENDA 	  50
#define WXCLIPS_LIST_RULES	  51
#define WXCLIPS_LIST_FACTS	  52
#define WXCLIPS_LIST_MATCHES	  53
#define WXCLIPS_LIST_BREAKPOINTS  54
#define WXCLIPS_LIST_TEMPLATES	  55
#define WXCLIPS_LIST_GLOBALS	  56
#define WXCLIPS_LIST_STRATEGY	  57
#define WXCLIPS_CLEAR_SCREEN      58

// Help menu
#define WXCLIPS_ABOUT             70
#define WXCLIPS_REFERENCE         72
#define WXCLIPS_CLIPS_HELP        73

void wxShowClipsWindow(void);
void wxDeleteClipsWindow(void);
void wxInitializeClips(void);
wxFrame *wxStartApp(void);
void wxClipsWorkProc(wxApp *app);

/*
 * Toolbar (Windows only so far)
 *
 */
 
#ifdef wx_msw
class wxClipsToolBar: public wxButtonBar
{
  public:
  wxClipsToolBar(wxFrame *frame, int x = 0, int y = 0, int w = -1, int h = -1,
            long style = 0, int direction = wxVERTICAL, int RowsOrColumns = 2);
  Bool OnLeftClick(int toolIndex, Bool toggled);
  void OnMouseEnter(int toolIndex);
  void OnPaint(void);
};

// Toolbar ids
#define DEVELOPMENT_TOOLBAR_LOAD_FILE       1
#define DEVELOPMENT_TOOLBAR_HELP            2
#define DEVELOPMENT_TOOLBAR_BATCH           3
#define DEVELOPMENT_TOOLBAR_CLEAR           4
#define DEVELOPMENT_TOOLBAR_LAUNCH          5
#define DEVELOPMENT_TOOLBAR_RESET           6
#define DEVELOPMENT_TOOLBAR_RUN             7
#define DEVELOPMENT_TOOLBAR_COPY            8
#endif // wx_msw

// wxCLIPS DDE server implementation (Windows only)
#ifdef wx_msw
class IDEConnection: public wxConnection
{
 public:
  IDEConnection(void);
  ~IDEConnection(void);
  Bool OnExecute(char *topic, char *data, int size, int format);
  char *OnRequest(char *topic, char *item, int *size, int format);
  Bool OnPoke(char *topic, char *item, char *data, int size, int format);
};

class IDEServer: public wxServer
{
 public:
  wxConnection *OnAcceptConnection(char *topic);
};
#endif

#endif // wxclipsh
