/*
 * wxcmdsp.h
 * Command library private prototypes: mainly callbacks.
 *
 */

#ifndef cmdsph
#define cmdsph

class wxClipsCallbacks: public wxList
{
 public:
  wxClipsCallbacks(void):wxList(wxKEY_STRING)
    {}
  ~wxClipsCallbacks(void)
   {
#ifdef USE_CLIPS
     wxNode *node = First();
     while (node)
     {
       delete[] (char *)node->Data();
       node = node->Next();
     }
#endif
   }
};

/*
 * Helper functions for per-object callbacks
 */
void wxClipsAddCallback(WXID id, char *eventType, wxClipsFunction callback);
void wxClipsRemoveCallback(WXID id, char *eventType);
wxClipsFunction wxClipsFindCallback(WXID id, char *eventType);
void wxClipsDeleteCallbacks(WXID id);

/*
 * Prototypes for per-class callbacks
 */
// wxcmisc.cc
void wxClipsTimerClassNotify(Bool *handled, WXID id);
void wxClipsObjectOnDelete(WXID id);

// wxcwin.cc
void wxClipsWindowOnSize(Bool *handled, WXID id, int w, int h);
Bool wxClipsWindowOnClose(Bool *handled, WXID id);
void wxClipsWindowOnChar(Bool *handled, WXID id, long keyCode, long eventId);
#ifdef wx_msw
Bool wxClipsWindowOnCharHook(Bool *handled, WXID id, long eventId);
#endif
void wxClipsWindowOnEvent(Bool *handled, WXID id, long eventId);
void wxClipsWindowOnScroll(Bool *handled, WXID id, long eventId);
void wxClipsWindowOnPaint(Bool *handled, WXID id);
void wxClipsPanelOnCommand(Bool *handled, WXID id, long itemId, long eventId);
void wxClipsFrameOnMenuCommand(Bool *handled, WXID id, int commandId);
void wxClipsFrameOnMenuSelect(Bool *handled, WXID id, int commandId);
void wxClipsFrameOnActivate(Bool *handled, WXID id, Bool active);
void wxClipsPanelOnDefaultAction(Bool *handled, WXID panelId, WXID itemId);

// wxctbar.cc
Bool wxClipsToolBarClassOnLeftClick(Bool *handled, WXID id, int toolIndex, Bool toggleDown);
void wxClipsToolBarClassOnRightClick(Bool *handled, WXID id, int toolIndex, float x, float y);
void wxClipsToolBarClassOnMouseEnter(Bool *handled, WXID id, int toolIndex);

// wxcitems.cc
void wxClipsItemCallback(Bool *handled, WXID id, WXID eventId);

// wxcdde.cc
Bool wxClipsConnectionOnAdvise(Bool *handled, WXID id, char *topic, char *item, char *data, int size, int format);
Bool wxClipsConnectionOnExecute(Bool *handled, WXID id, char *topic, char *data, int size, int format);
Bool wxClipsConnectionOnPoke(Bool *handled, WXID id, char *topic, char *item, char *data, int size, int format);
char *wxClipsConnectionOnRequest(Bool *handled, WXID id, char *topic, char *item, int *size, int format);
Bool wxClipsConnectionOnStartAdvise(Bool *handled, WXID id, char *topic, char *item);
Bool wxClipsConnectionOnStopAdvise(Bool *handled, WXID id, char *topic, char *item);
Bool wxClipsServerOnAcceptConnection(Bool *handled, WXID id, char *topic, WXID connectionId);

// wxcgrid.cc
#if defined(wx_msw) && USE_WXGRID
void wxClipsGridOnCellLeftClick(Bool *handled, WXID id, int row, int col, int x, int y, Bool controlDown, Bool shiftDown);
void wxClipsGridOnCellRightClick(Bool *handled, WXID id, int row, int col, int x, int y, Bool controlDown, Bool shiftDown);
void wxClipsGridOnCellChange(Bool *handled, WXID id, int row, int col);
void wxClipsGridOnChangeLabels(Bool *handled, WXID id);
void wxClipsGridOnChangeSelectionLabel(Bool *handled, WXID id);
#endif

#endif // cmdsph

