/*
 * wxcgdi.cc
 * wxWindows command processing: GDI objects
 *
 */

// For compilers that support precompilation, includes "wx.h".
#include "wx_prec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx.h"
#endif

#include "wx_hash.h"
#include "wx_types.h"

#ifdef wx_msw
// #if LOADABLE_BITMAPS
// #include "dib.h"
// #endif
// #if LOADABLE_ICONS
// #include "wxcurico.h"
// #endif
#endif
#if defined(wx_x)// && USE_IMAGE_LOADING_IN_X
#include "../../image/src/wx_image.h"
#endif

#include <iostream.h>
#include <ctype.h>
#include <stdlib.h>

#include "wxextend.h"

extern wxObject *wxGetTypedObject(long objectId, WXTYPE expectedType);
extern void wxStoreThingInTable(long item_id, wxObject *object, WXTYPE type);
extern void wxDeleteThingInTable(long item_id);
extern wxObject *wxGetThingFromTable(long item_id, WXTYPE *type);
extern void wxCleanWindows(void);

extern ExtensionErrorClass ExtensionError;

// Pen
class wxExtensionPen: public wxPen
{
 public:
  wxExtensionPen(wxColour& col, int width, int style):
   wxPen(col, width, style)
   {
      wxStoreThingInTable((long)this, this, CLASSINFO(wxPen));
   }
                   
  ~wxExtensionPen()
   {
      wxExtensionFunction func = wxFindExtensionCallback(CLASSINFO(wxPen), "OnDelete");
      if (func)
        (* (void (*)(long)) func) ((long)this);

      wxDeleteThingInTable((long)this);
   }
};

// Brush
class wxExtensionBrush: public wxBrush
{
 public:
  wxExtensionBrush(wxColour& col, int style):
   wxBrush(col, style)
   {
      wxStoreThingInTable((long)this, this, CLASSINFO(wxBrush));
   }
                   
  ~wxExtensionBrush()
   {
      wxExtensionFunction func = wxFindExtensionCallback(CLASSINFO(wxBrush), "OnDelete");
      if (func)
        (* (void (*)(long)) func) ((long)this);
      wxDeleteThingInTable((long)this);
   }
};

// Font
class wxExtensionFont: public wxFont
{
 public:
  wxExtensionFont(int pointSize, int family, int style, int weight, Bool underlined):
   wxFont(pointSize, family, style, weight, underlined)
   {
      wxStoreThingInTable((long)this, this, CLASSINFO(wxFont));
   }
                   
  ~wxExtensionFont()
   {
      wxExtensionFunction func = wxFindExtensionCallback(CLASSINFO(wxFont), "OnDelete");
      if (func)
        (* (void (*)(long)) func) ((long)this);
      wxDeleteThingInTable((long)this);
   }
};

// Icon
class wxExtensionIcon: public wxIcon
{
 public:
  wxExtensionIcon(char *file, long typ):wxIcon(file, typ)
   {
      wxStoreThingInTable((long)this, this, CLASSINFO(wxIcon));
   }
                   
  ~wxExtensionIcon()
   {
      wxExtensionFunction func = wxFindExtensionCallback(CLASSINFO(wxIcon), "OnDelete");
      if (func)
        (* (void (*)(long)) func) ((long)this);
      wxDeleteThingInTable((long)this);
   }
};

// Create a new pen
WXID wxcPenCreate(char *colour, int width, int style)
{
  if (!colour)
    return -1;

  wxColour *col = wxTheColourDatabase->FindColour(colour);
  if (!col)
  {
    ExtensionError << "Error in pen-create: could not find colour " << colour << ".\n";
    return 0;
  }
  wxExtensionPen *pen = new wxExtensionPen(*col, width, style);
  return (long)pen;
}

WXID wxcPenCreate(WXCOLOUR colour, int width, int style)
{
  wxColour col((unsigned char)wxcColourRed(colour), (unsigned char)wxcColourGreen(colour), (unsigned char)wxcColourBlue(colour));
  wxExtensionPen *pen = new wxExtensionPen(col, width, style);
  return (long)pen;
}

// Create a new pen (string version)
WXID wxcPenCreate(char *colour, int width, char *style)
{
  if (!colour || !style)
    return -1;

  int sty = wxSOLID;
  if (strcmp(style, "wxSOLID") == 0)
    sty = wxSOLID;
  else if (strcmp(style, "wxDOT") == 0)
    sty = wxDOT;
  else if (strcmp(style, "wxLONG_DASH") == 0)
    sty = wxLONG_DASH;
  else if (strcmp(style, "wxSHORT_DASH") == 0)
    sty = wxSHORT_DASH;
  else if (strcmp(style, "wxDOT_DASH") == 0)
    sty = wxDOT_DASH;
  else if (strcmp(style, "wxTRANSPARENT") == 0)
    sty = wxTRANSPARENT;
  else
  {
    ExtensionError << "Error in pen-create: unrecognized style " << style << "\n";
    ExtensionError << "Style must be one of wxSOLID, wxDOT, wxLONG_DASH, wxSHORT_DASH, wxDOT_DASH, wxTRANSPARENT.\n";
    return 0;
  }
  return wxcPenCreate(colour, width, sty);
}

// Create a new pen (string version)
WXID wxcPenCreate(WXCOLOUR colour, int width, char *style)
{
  if (!style)
    return -1;

  int sty = wxSOLID;
  if (strcmp(style, "wxSOLID") == 0)
    sty = wxSOLID;
  else if (strcmp(style, "wxDOT") == 0)
    sty = wxDOT;
  else if (strcmp(style, "wxLONG_DASH") == 0)
    sty = wxLONG_DASH;
  else if (strcmp(style, "wxSHORT_DASH") == 0)
    sty = wxSHORT_DASH;
  else if (strcmp(style, "wxDOT_DASH") == 0)
    sty = wxDOT_DASH;
  else if (strcmp(style, "wxTRANSPARENT") == 0)
    sty = wxTRANSPARENT;
  else
  {
    ExtensionError << "Error in pen-create: unrecognized style " << style << "\n";
    ExtensionError << "Style must be one of wxSOLID, wxDOT, wxLONG_DASH, wxSHORT_DASH, wxDOT_DASH, wxTRANSPARENT.\n";
    return 0;
  }
  return wxcPenCreate(colour, width, sty);
}

// Delete pen
Bool wxcPenDelete(WXID id)
{
  wxPen *pen = (wxPen *)wxGetTypedObject(id, CLASSINFO(wxPen));
  if (pen)
  {
    delete pen;
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in pen-delete: could not find pen " << id << ".\n";
    return FALSE;
  }
}

// Create a new brush
WXID wxcBrushCreate(char *colour, int style)
{
  if (!colour)
    return -1;

  wxColour *col = wxTheColourDatabase->FindColour(colour);
  if (!col)
  {
    ExtensionError << "Error in brush-create: could not find colour " << colour << ".\n";
    return 0;
  }
  wxExtensionBrush *brush = new wxExtensionBrush(*col, style);
  return (long)brush;
}

WXID wxcBrushCreate(WXCOLOUR colour, int style)
{
  wxColour col((unsigned char)wxcColourRed(colour), (unsigned char)wxcColourGreen(colour), (unsigned char)wxcColourBlue(colour));
  wxExtensionBrush *brush = new wxExtensionBrush(col, style);
  return (long)brush;
}

// Create a new brush (string version)
long wxcBrushCreate(char *colour, char *style)
{
  if (!colour || !style)
    return -1;

  int sty = 0;
  if (strcmp(style, "wxSOLID") == 0)
    sty = wxSOLID;
  else if (strcmp(style, "wxTRANSPARENT") == 0)
    sty = wxTRANSPARENT;
  else
  {
    ExtensionError << "Error in brush-create: unrecognized style " << style << "\n";
    ExtensionError << "Style must be one of wxSOLID, wxTRANSPARENT.\n";
    return 0;
  }
  return wxcBrushCreate(colour, sty);
}

long wxcBrushCreate(WXCOLOUR colour, char *style)
{
  if (!style)
    return -1;

  int sty = 0;
  if (strcmp(style, "wxSOLID") == 0)
    sty = wxSOLID;
  else if (strcmp(style, "wxTRANSPARENT") == 0)
    sty = wxTRANSPARENT;
  else
  {
    ExtensionError << "Error in brush-create: unrecognized style " << style << "\n";
    ExtensionError << "Style must be one of wxSOLID, wxTRANSPARENT.\n";
    return 0;
  }
  return wxcBrushCreate(colour, sty);
}

// Delete pen
Bool wxcBrushDelete(WXID id)
{
  wxBrush *brush = (wxBrush *)wxGetTypedObject(id, CLASSINFO(wxBrush));
  if (brush)
  {
    delete brush;
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in brush-delete: could not find brush " << id << ".\n";
    return FALSE;
  }
}

// Create a new font
WXID wxcFontCreate(int pointSize, int family, int style, int weight,
  Bool underlined)
{
  wxExtensionFont *font = new wxExtensionFont(pointSize, family, style, weight, underlined);
  return (long)font;
}

// Create a new font (string version)
long wxcFontCreate(int pointSize, char *family, char *style, char *weight,
  Bool underlined)
{
  int wxStyle = 0;
  int wxFamily = 0;
  int wxWeight = 0;
  if (!family || !style || !weight)
    return -1;

  if (strcmp(family, "wxROMAN") == 0)
    wxFamily = wxROMAN;
  else if (strcmp(family, "wxSCRIPT") == 0)
    wxFamily = wxSCRIPT;
  else if (strcmp(family, "wxDECORATIVE") == 0)
    wxFamily = wxDECORATIVE;
  else if (strcmp(family, "wxSWISS") == 0)
    wxFamily = wxSWISS;
  else if (strcmp(family, "wxMODERN") == 0)
    wxFamily = wxMODERN;
  else if (strcmp(family, "wxDEFAULT") == 0)
    wxFamily = wxMODERN;
  else
  {
    ExtensionError << "Error in font-create: unrecognized family " << family << ".\n";
    ExtensionError << "Family must be one of wxDECORATIVE, wxDEFAULT, wxMODERN, wxROMAN, wxSCRIPT, wxSWISS.\n";
    return -1;
  }

  if (strcmp(weight, "wxNORMAL") == 0)
    wxWeight = wxNORMAL;
  else if (strcmp(weight, "wxBOLD") == 0)
    wxWeight = wxBOLD;
  else if (strcmp(weight, "wxLIGHT") == 0)
    wxWeight = wxLIGHT;
  else
  {
    ExtensionError << "Error in font-create: unrecognized weight " << weight << ".\n";
    ExtensionError << "Weight must be one of wxNORMAL, wxBOLD, wxLIGHT.\n";
    return -1;
  }

  if (strcmp(style, "wxNORMAL") == 0)
    wxStyle = wxNORMAL;
  else if (strcmp(style, "wxITALIC") == 0)
    wxStyle = wxITALIC;
  else
  {
    ExtensionError << "Error in font-create: unrecognized style " << style << ".\n";
    ExtensionError << "Style must be one of wxNORMAL, wxITALIC.\n";
    return -1;
  }
  return wxcFontCreate(pointSize, wxFamily, wxStyle, wxWeight, underlined);
}

// Delete font
Bool wxcFontDelete(WXID id)
{
  wxFont *font = (wxFont *)wxGetTypedObject(id, CLASSINFO(wxFont));
  if (font)
  {
    delete font;
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in font-delete: could not find font " << id << ".\n";
    return FALSE;
  }
}

// Create a new bitmap
long wxcBitmapCreate(int width, int height, int depth)
{
  wxBitmap *bitmap = new wxBitmap(width, height, depth);
  wxStoreThingInTable((long)bitmap, bitmap, CLASSINFO(wxBitmap));
  return (long)bitmap;
}

// Load a bitmap from a file (.XBM for X, .BMP for Windows)
long wxcBitmapLoadFromFile(char *f, long bitmapType)
{
  wxBitmap *newBitmap = new wxBitmap(f, bitmapType);
  if (!newBitmap->Ok())
  {
    delete newBitmap;
    ExtensionError << "Error in bitmap-load-from-file: error while loading bitmap " << f << ".\n";
    return -1;
  }

  wxStoreThingInTable((long)newBitmap, newBitmap, CLASSINFO(wxBitmap));
  return (long)newBitmap;
}

// Load a bitmap from a file (.XBM for X, .BMP for Windows)
// String version
long wxcBitmapLoadFromFile(char *f, char *bitmapType)
{
  long bitmapTypeInt = wxExtensionParseBitList(bitmapType);
  if (bitmapTypeInt == 0)
  {
    ExtensionError << "Error in bitmap-load-from-file: unrecognized bitmap type " << bitmapType << ".\n";
    return 0;
  }

  return wxcBitmapLoadFromFile(f, bitmapTypeInt);
}

// Delete bitmap
Bool wxcBitmapDelete(WXID id)
{
  wxBitmap *bitmap = (wxBitmap *)wxGetTypedObject(id, CLASSINFO(wxBitmap));
  if (bitmap)
  {
    wxDeleteThingInTable(id);
    if (bitmap->GetColourMap())
      wxDeleteThingInTable((long)bitmap->GetColourMap());

    delete bitmap;
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in bitmap-delete: could not find bitmap " << id << ".\n";
    return FALSE;
  }
}

// Get the bitmap's width
int wxcBitmapGetWidth(WXID id)
{
  wxBitmap *bitmap = (wxBitmap *)wxGetTypedObject(id, CLASSINFO(wxBitmap));
  if (bitmap)
  {
    return bitmap->GetWidth();
  }
  else
  {
    ExtensionError << "Error in bitmap-get-width: could not find bitmap " << id << ".\n";
    return 0;
  }
}

// Get the bitmap's height
int wxcBitmapGetHeight(WXID id)
{
  wxBitmap *bitmap = (wxBitmap *)wxGetTypedObject(id, CLASSINFO(wxBitmap));
  if (bitmap)
  {
    return bitmap->GetHeight();
  }
  else
  {
    ExtensionError << "Error in bitmap-get-height: could not find bitmap " << id << ".\n";
    return 0;
  }
}

// Get the bitmap's depth
int wxcBitmapGetDepth(WXID id)
{
  wxBitmap *bitmap = (wxBitmap *)wxGetTypedObject(id, CLASSINFO(wxBitmap));
  if (bitmap)
  {
    return bitmap->GetDepth();
  }
  else
  {
    ExtensionError << "Error in bitmap-get-depth: could not find bitmap " << id << ".\n";
    return 0;
  }
}

// Get the bitmap's colourmap (if any)
WXID wxcBitmapGetColourMap(WXID id)
{
  wxBitmap *bitmap = (wxBitmap *)wxGetTypedObject(id, CLASSINFO(wxBitmap));
  if (bitmap)
  {
    wxColourMap *cmap = bitmap->GetColourMap();
    if (!cmap)
      return 0;
    else
    {
      long cmapId = (long)cmap;
      if (!wxGetTypedObject(cmapId, CLASSINFO(wxColourMap)))
        wxStoreThingInTable((long)cmapId, cmap, CLASSINFO(wxColourMap));
      return cmapId;
    }
  }
  else
  {
    ExtensionError << "Error in bitmap-get-colourmap: could not find bitmap " << id << ".\n";
    return 0;
  }
}

// Create a new icon
WXID wxcIconCreate(char *file)
{
  if (!file)
    return -1;

  wxExtensionIcon *icon = new wxExtensionIcon(file, wxBITMAP_TYPE_ICO);
  return (long)icon;
}

// Load an icon from a file (.XBM for X, .ICO for Windows)
WXID wxcIconLoadFromFile(char *f, char *bitmapType)
{
  if (FileExists(f))
  {
    if (!bitmapType)
#ifdef wx_x
      bitmapType = "wxBITMAP_TYPE_XBM";
#else
      bitmapType = "wxBITMAP_TYPE_ICO";
#endif
    long bitmapTypeInt = wxExtensionParseBitList(bitmapType);
    if (bitmapTypeInt == 0)
    {
      ExtensionError << "Error in icon-load-from-file: unrecognized bitmap type " << bitmapType << ".\n";
      return 0;
    }

    wxExtensionIcon *icon = new wxExtensionIcon(f, bitmapTypeInt);
    return (long)icon;
  }
  else
  {
    ExtensionError << "Error in icon-load-from-file: could not find icon file " << f << ".\n";
    return 0;
  }
  return 0;
}

// Delete icon
Bool wxcIconDelete(WXID id)
{
  wxIcon *icon = (wxIcon *)wxGetTypedObject(id, CLASSINFO(wxIcon));
  if (icon)
  {
//    wxDeleteThingInTable(id);
    delete icon;
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in icon-delete: could not find icon " << id << ".\n";
    return FALSE;
  }
}

// Get the icon's width
int wxcIconGetWidth(WXID id)
{
  wxIcon *icon = (wxIcon *)wxGetTypedObject(id, CLASSINFO(wxIcon));
  if (icon)
  {
    return icon->GetWidth();
  }
  else
  {
    ExtensionError << "Error in icon-get-width: could not find icon " << id << ".\n";
    return 0;
  }
}

// Get the icon's height
int wxcIconGetHeight(WXID id)
{
  wxIcon *icon = (wxIcon *)wxGetTypedObject(id, CLASSINFO(wxIcon));
  if (icon)
  {
    return icon->GetHeight();
  }
  else
  {
    ExtensionError << "Error in icon-get-height: could not find icon " << id << ".\n";
    return 0;
  }
}

/*
 * Cursor
 */

wxExtensionBitListStruct wxExtensionCursorTable[] =
{
 { "wxCURSOR_ARROW", wxCURSOR_ARROW },
 { "wxCURSOR_BULLSEYE", wxCURSOR_BULLSEYE },
 { "wxCURSOR_CHAR", wxCURSOR_CHAR },
 { "wxCURSOR_CROSS", wxCURSOR_CROSS },
 { "wxCURSOR_HAND", wxCURSOR_HAND },
 { "wxCURSOR_IBEAM", wxCURSOR_IBEAM },
 { "wxCURSOR_LEFT_BUTTON", wxCURSOR_LEFT_BUTTON },
 { "wxCURSOR_MAGNIFIER", wxCURSOR_MAGNIFIER },
 { "wxCURSOR_MIDDLE_BUTTON", wxCURSOR_MIDDLE_BUTTON },
 { "wxCURSOR_NO_ENTRY", wxCURSOR_NO_ENTRY },
 { "wxCURSOR_PAINT_BRUSH", wxCURSOR_PAINT_BRUSH },
 { "wxCURSOR_PENCIL", wxCURSOR_PENCIL },
 { "wxCURSOR_POINT_LEFT", wxCURSOR_POINT_LEFT },
 { "wxCURSOR_POINT_RIGHT", wxCURSOR_POINT_RIGHT },
 { "wxCURSOR_QUESTION_ARROW", wxCURSOR_QUESTION_ARROW },
 { "wxCURSOR_RIGHT_BUTTON", wxCURSOR_RIGHT_BUTTON },
 { "wxCURSOR_SIZENESW", wxCURSOR_SIZENESW },
 { "wxCURSOR_SIZENS", wxCURSOR_SIZENS },
 { "wxCURSOR_SIZENWSE", wxCURSOR_SIZENWSE },
 { "wxCURSOR_SIZEWE", wxCURSOR_SIZEWE },
 { "wxCURSOR_SIZING", wxCURSOR_SIZING },
 { "wxCURSOR_SPRAYCAN", wxCURSOR_SPRAYCAN },
 { "wxCURSOR_WAIT", wxCURSOR_WAIT },
 { "wxCURSOR_WATCH", wxCURSOR_WATCH },
 { "wxCURSOR_BLANK", wxCURSOR_BLANK },
#ifdef wx_x
  /* Not yet implemented for Windows */
 { "wxCURSOR_CROSS_REVERSE", wxCURSOR_CROSS_REVERSE },
 { "wxCURSOR_DOUBLE_ARROW", wxCURSOR_DOUBLE_ARROW },
 { "wxCURSOR_BASED_ARROW_UP", wxCURSOR_BASED_ARROW_UP },
 { "wxCURSOR_BASED_ARROW_DOWN", wxCURSOR_BASED_ARROW_DOWN },
#endif
 { "wxCURSOR_BLANK", wxCURSOR_BLANK }
};

int wxExtensionCursorCount = (sizeof(wxExtensionCursorTable)/sizeof(wxExtensionBitListStruct));

static long wxExtensionParseGDIName(char *name, wxExtensionBitListStruct table[], int n)
{
  for (int i = 0; i < n; i++)
    if (strcmp(name, table[i].word) == 0)
      return table[i].bits;
  return -1;
}

// Create a new stock cursor
WXID wxcCursorCreate(char *cursorType)
{
  long intCursorType = wxExtensionParseGDIName(cursorType, wxExtensionCursorTable, wxExtensionCursorCount);
  if (intCursorType == -1)
  {
    ExtensionError << "Error in cursor-create: " << cursorType << " is not a know stock cursor name.\n";
    return 0;
  }

  wxCursor *cursor = new wxCursor((int)intCursorType);
  wxStoreThingInTable((long)cursor, cursor, CLASSINFO(wxCursor));
  return (long)cursor;
}

// Create a new stock cursor (integer version)
WXID wxcCursorCreate(int cursorType)
{
  wxCursor *cursor = new wxCursor(cursorType);
  wxStoreThingInTable((long)cursor, cursor, CLASSINFO(wxCursor));
  return (long)cursor;
}

// Load a cursor from a file (.XBM for X, .CUR for Windows)
WXID wxcCursorLoadFromFile(char *f, char *bitmapType, int x, int y)
{
  if (FileExists(f))
  {
    if (!bitmapType)
#ifdef wx_x
      bitmapType = "wxBITMAP_TYPE_XBM";
#else
      bitmapType = "wxBITMAP_TYPE_CUR";
#endif
    long bitmapTypeInt = wxExtensionParseBitList(bitmapType);
    if (bitmapTypeInt == 0)
    {
      ExtensionError << "Error in cursor-load-from-file: unrecognized bitmap type " << bitmapType << ".\n";
      return 0;
    }

    wxCursor *cursor = new wxCursor(f, bitmapTypeInt, x, y);

    wxStoreThingInTable((long)cursor, cursor, CLASSINFO(wxCursor));
    return (long)cursor;
  }
  else
  {
    ExtensionError << "Error in cursor-load-from-file: could not find cursor file " << f << ".\n";
    return 0;
  }
  return 0;
}

// Delete cursor
Bool wxcCursorDelete(WXID id)
{
  wxCursor *cursor = (wxCursor *)wxGetTypedObject(id, CLASSINFO(wxCursor));
  if (cursor)
  {
    wxDeleteThingInTable(id);
    delete cursor;
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in cursor-delete: could not find cursor " << id << ".\n";
    return FALSE;
  }
}

// Colours: colour is stored inside a long integer, not a class.
WXCOLOUR wxcColourCreate(int red, int green, int blue)
{
  return ((WXCOLOUR)(((unsigned char)(red)|((int)(green)<<8))|(((long)(unsigned char)(blue))<<16)));
}

WXCOLOUR wxcColourCreate(char *colour)
{
  wxColour *col = wxTheColourDatabase->FindColour((const char *)colour);
  if (col)
  {
    int red = (int)col->Red();
    int green = (int)col->Green();
    int blue = (int)col->Blue();
    
    return ((WXCOLOUR)(((unsigned char)(red)|((int)(green)<<8))|(((long)(unsigned char)(blue))<<16)));
  }
  else
    return 0;
}

int wxcColourRed(WXCOLOUR col)
{
  return (int)(unsigned char)col;
}

int wxcColourGreen(WXCOLOUR col)
{
  return (int)((unsigned char)(((int)(col)) >> 8));
}

int wxcColourBlue(WXCOLOUR col)
{
  return (int)((unsigned char)((col)>>16));
}
