/*
 * exgrid.cc
 * wxExtend grid class
 *
 */

// For compilers that support precompilation, includes "wx.h".
#include "wx_prec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx.h"
#endif

#if defined(wx_msw) && USE_WXGRID

#include "wxgrid.h"
#include "wxstring.h"

#include "wxextend.h"

class wxExtensionGrid: public wxGrid
{
 public:
  wxExtensionGrid (wxWindow * parent,
		    int x = -1, int y = -1,
		    int width = -1, int height = -1, long style = 0, char *name = "grid"):
    wxGrid (parent, x, y, width, height, style, name)
  {
    wxStoreThingInTable ((long) this, this, CLASSINFO(wxGenericGrid));
    wxCanvasDC *dc = GetDC ();
    if (dc)
      wxStoreThingInTable ((long) dc, dc, CLASSINFO(wxCanvasDC));
    wxText *text = GetTextItem();
    if (text)
      wxStoreThingInTable ((long) text, text, CLASSINFO(wxText));
  }
  wxExtensionGrid (void)
  {
    wxStoreThingInTable ((long) this, this, CLASSINFO(wxGenericGrid));
  }

   ~wxExtensionGrid ()
  {
    wxExtensionFunction func = wxFindExtensionCallback (CLASSINFO(wxGenericGrid), "OnDelete");
    if (func)
      (*(void (*)(long)) func) ((long) this);

    wxDeleteThingInTable ((long) this);
    wxCanvasDC *dc = GetDC ();
    wxDeleteThingInTable ((long) dc);
    wxText *text = GetTextItem();
    if (text)
      wxDeleteThingInTable ((long) text);
  }

  void OnSize (int w, int h);
  void OnPaint (void);
  void OnCellLeftClick(int row, int col, int x, int y, Bool controlDown, Bool shiftDown);
  void OnCellRightClick(int row, int col, int x, int y, Bool controlDown, Bool shiftDown);
  void OnChangeLabels(void);
  void OnCellChange(int row, int col);
  void OnChangeSelectionLabel(void);
};

void wxExtensionGrid::OnSize (int w, int h)
{
  wxExtensionFunction func = wxFindExtensionCallback (CLASSINFO(wxGenericGrid), "OnSize");
  if (func)
    {
      Bool handled = FALSE;
      (*(void (*)(Bool *, long, int, int)) func) (&handled, (long) this, w, h);

      if (!handled)
        wxGrid::OnSize (w, h);
    }
  else
    wxGrid::OnSize (w, h);
}

void wxExtensionGrid::OnPaint (void)
{
  wxExtensionFunction func = wxFindExtensionCallback (CLASSINFO(wxGenericGrid), "OnPaint");
  if (func)
    {
      Bool handled = FALSE;
      (*(void (*)(Bool *, long)) func) (&handled, (long) this);

      if (!handled)
        wxGrid::OnPaint ();
    }
  else
    wxGrid::OnPaint ();
}

void wxExtensionGrid::OnChangeLabels (void)
{
  wxExtensionFunction func = wxFindExtensionCallback (CLASSINFO(wxGenericGrid), "OnChangeLabels");
  if (func)
    {
      Bool handled = FALSE;
      (*(void (*)(Bool *, long)) func) (&handled, (long) this);

      if (!handled)
        wxGrid::OnChangeLabels ();
    }
  else
    wxGrid::OnChangeLabels ();
}

void wxExtensionGrid::OnChangeSelectionLabel (void)
{
  wxExtensionFunction func = wxFindExtensionCallback (CLASSINFO(wxGenericGrid), "OnChangeSelectionLabel");
  if (func)
    {
      Bool handled = FALSE;
      (*(void (*)(Bool *, long)) func) (&handled, (long) this);

      if (!handled)
        wxGrid::OnChangeSelectionLabel ();
    }
  else
    wxGrid::OnChangeSelectionLabel ();
}

void wxExtensionGrid::OnCellLeftClick(int row, int col, int x, int y, Bool controlDown, Bool shiftDown)
{
  wxExtensionFunction func = wxFindExtensionCallback (CLASSINFO(wxGenericGrid), "OnCellLeftClick");
  if (func)
    {
      Bool handled = FALSE;
      (*(void (*)(Bool *, long, int, int, int, int, Bool, Bool)) func) (&handled, (long) this, row, col, x, y, controlDown, shiftDown);

      if (!handled)
        wxGrid::OnCellLeftClick(row, col, x, y, controlDown, shiftDown);
    }
  else
    wxGrid::OnCellLeftClick(row, col, x, y, controlDown, shiftDown);
}

void wxExtensionGrid::OnCellRightClick(int row, int col, int x, int y, Bool controlDown, Bool shiftDown)
{
  wxExtensionFunction func = wxFindExtensionCallback (CLASSINFO(wxGenericGrid), "OnCellRightClick");
  if (func)
    {
      Bool handled = FALSE;
      (*(void (*)(Bool *, long, int, int, int, int, Bool, Bool)) func) (&handled, (long) this, row, col, x, y, controlDown, shiftDown);

      if (!handled)
        wxGrid::OnCellRightClick(row, col, x, y, controlDown, shiftDown);
    }
  else
    wxGrid::OnCellRightClick(row, col, x, y, controlDown, shiftDown);
}

void wxExtensionGrid::OnCellChange(int row, int col)
{
  wxExtensionFunction func = wxFindExtensionCallback (CLASSINFO(wxGenericGrid), "OnCellChange");
  if (func)
    {
      Bool handled = FALSE;
      (*(void (*)(Bool *, long, int, int)) func) (&handled, (long) this, row, col);

      if (!handled)
        wxGrid::OnCellChange(row, col);
    }
  else
    wxGrid::OnCellChange(row, col);
}

// Create a new grid
long wxcGridCreate (long parentId, int x, int y, int width, int height, long style, char *name)
{
  wxWindow *parent = NULL;

  if (parentId != 0)
    {
      parent = (wxWindow *) wxGetTypedObject (parentId, CLASSINFO(wxWindow));
      if (!parent)
	{
	  ExtensionError << "Error in grid-create: could not find parent window " << parentId << ".\n";
	  return 0;
	}
    }

  wxExtensionGrid *grid = new wxExtensionGrid (parent, (int) x, (int) y, (int) width, (int) height, style, name);
  return (long) grid;
}

// Grid creation (string version)
long wxcGridCreate (long parentId, int x, int y, int width, int height, char *styleString, char *name)
{
  wxWindow *parent = NULL;

  if (parentId != 0)
    {
      parent = (wxWindow *) wxGetTypedObject (parentId, CLASSINFO(wxWindow));
      if (!parent)
	{
	  ExtensionError << "Error in grid-create: could not find parent window " << parentId << ".\n";
	  return 0;
	}
    }
  long style = wxExtensionParseBitList (styleString);

  wxExtensionGrid *grid = new wxExtensionGrid (parent, (int) x, (int) y, (int) width, (int) height, style, name);
  return (long) grid;
}

Bool wxcGridCreateGrid (WXID id, int cols, int rows, int defaultWidth, int defaultHeight)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-create-grid: could not find grid " << id << ".\n";
    return FALSE;
  }
  return grid->CreateGrid(rows, cols, NULL, NULL, (short)defaultWidth, (short)defaultHeight);
}

void wxcGridClearGrid (WXID id)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-clear-grid: could not find grid " << id << ".\n";
    return;
  }
  grid->ClearGrid();
}

Bool wxcGridInsertCols (WXID id, int pos, int n, Bool updateLabels)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-insert-cols: could not find grid " << id << ".\n";
    return FALSE;
  }
  return grid->InsertCols(pos, n, updateLabels);
}

Bool wxcGridInsertRows (WXID id, int pos, int n, Bool updateLabels)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-insert-rows: could not find grid " << id << ".\n";
    return FALSE;
  }
  return grid->InsertRows(pos, n, updateLabels);
}

Bool wxcGridAppendCols (WXID id, int n, Bool updateLabels)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-append-cols: could not find grid " << id << ".\n";
    return FALSE;
  }
  return grid->AppendCols(n, updateLabels);
}

Bool wxcGridAppendRows (WXID id, int n, Bool updateLabels)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-append-rows: could not find grid " << id << ".\n";
    return FALSE;
  }
  return grid->AppendRows(n, updateLabels);
}

Bool wxcGridDeleteCols (WXID id, int pos, int n, Bool updateLabels)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-delete-cols: could not find grid " << id << ".\n";
    return FALSE;
  }
  return grid->DeleteCols(pos, n, updateLabels);
}

Bool wxcGridDeleteRows (WXID id, int pos, int n, Bool updateLabels)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-delete-rows: could not find grid " << id << ".\n";
    return FALSE;
  }
  return grid->DeleteRows(pos, n, updateLabels);
}

void wxcGridSetCellValue (WXID id, char *value, int row, int col)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-set-cell-value: could not find grid " << id << ".\n";
    return;
  }
  grid->SetCellValue(value, row, col);
}

char *wxcGridGetCellValue (WXID id, int row, int col)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-get-cell-value: could not find grid " << id << ".\n";
    return NULL;
  }
  return grid->GetCellValue(row, col).GetData();
}

void wxcGridSetCellBitmap (WXID id, WXID bitmapId, int row, int col)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-set-cell-bitmap: could not find grid " << id << ".\n";
    return;
  }
  wxBitmap *bitmap = (wxBitmap *)wxGetTypedObject(id, CLASSINFO(wxBitmap));
  if (!bitmap)
  {
    ExtensionError << "Error in grid-set-cell-bitmap: could not find bitmap " << bitmapId << ".\n";
    return;
  }
  grid->SetCellBitmap(bitmap, row, col);
}

WXID wxcGridGetCellBitmap (WXID id, int row, int col)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-get-cell-bitmap: could not find grid " << id << ".\n";
    return 0;
  }
  return (WXID)grid->GetCellBitmap(row, col);
}

void wxcGridSetCellAlignment (WXID id, int flag, int row, int col)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-set-cell-alignment: could not find grid " << id << ".\n";
    return;
  }
  if (row == -1 && col == -1)
    grid->SetCellAlignment(flag);
  else
    grid->SetCellAlignment(flag, row, col);
}

void wxcGridSetCellAlignment (WXID id, char *flag, int row, int col)
{
  int flag2 = 0;
  wxString str(flag);
  if (str == "wxLEFT")
    flag2 = wxLEFT;
  else if (str == "wxRIGHT")
    flag2 = wxRIGHT;
  else if (str == "wxCENTRE")
    flag2 = wxCENTRE;
  else
  {
    ExtensionError << "Error in grid-set-cell-alignment: alignment should be one of wxLEFT, wxRIGHT, wxCENTRE.\n";
    return;
  }
  
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-set-cell-alignment: could not find grid " << id << ".\n";
    return;
  }
  if (row == -1 && col == -1)
    grid->SetCellAlignment(flag2);
  else
    grid->SetCellAlignment(flag2, row, col);
}

int wxcGridGetCellAlignment (WXID id, int row, int col)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-get-cell-alignment: could not find grid " << id << ".\n";
    return wxLEFT;
  }
  if (row == -1 && col == -1)
    return grid->GetCellAlignment();
  else
    return grid->GetCellAlignment(row, col);
}

char *wxcGridGetCellAlignmentString (WXID id, int row, int col)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-get-cell-alignment: could not find grid " << id << ".\n";
    return "wxLEFT";
  }
  int flag = 0;

  if (row == -1 && col == -1)
    flag = grid->GetCellAlignment();
  else
    flag = grid->GetCellAlignment(row, col);
  switch (flag)
  {
    case wxLEFT:
      return "wxLEFT";
    case wxRIGHT:
      return "wxRIGHT";
    case wxCENTRE:
      return "wxCENTRE";
    default:
      return "wxLEFT";
  }
  return NULL;
}

void wxcGridSetCellTextColour (WXID id, WXCOLOUR colour, int row, int col)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-set-cell-text-colour: could not find grid " << id << ".\n";
    return;
  }
  wxColour colour1((unsigned char)wxcColourRed(colour), (unsigned char)wxcColourGreen(colour), (unsigned char)wxcColourBlue(colour));
  if (row == -1 && col == -1)
    grid->SetCellTextColour(colour1);
  else
    grid->SetCellTextColour(colour1, row, col);
}

WXCOLOUR wxcGridGetCellTextColour (WXID id, int row, int col)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-get-cell-text-colour: could not find grid " << id << ".\n";
    return 0;
  }
  if (row == -1 && col == -1)
    return wxcColourCreate(grid->GetCellTextColour().Red(), grid->GetCellTextColour().Green(), grid->GetCellTextColour().Blue());
  else
    return wxcColourCreate(grid->GetCellTextColour(row, col).Red(),
       grid->GetCellTextColour(row, col).Green(), grid->GetCellTextColour(row, col).Blue());
}

void wxcGridSetCellBackgroundColour (WXID id, WXCOLOUR colour, int row, int col)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-set-cell-background-colour: could not find grid " << id << ".\n";
    return;
  }
  wxColour colour1((unsigned char)wxcColourRed(colour), (unsigned char)wxcColourGreen(colour), (unsigned char)wxcColourBlue(colour));
  if (row == -1 && col == -1)
    grid->SetCellBackgroundColour(colour1);
  else
    grid->SetCellBackgroundColour(colour1, row, col);
}

WXCOLOUR wxcGridGetCellBackgroundColour (WXID id, int row, int col)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-get-cell-background-colour: could not find grid " << id << ".\n";
    return 0;
  }
  if (row == -1 && col == -1)
    return wxcColourCreate(grid->GetCellBackgroundColour().Red(), grid->GetCellBackgroundColour().Green(), grid->GetCellBackgroundColour().Blue());
  else
    return wxcColourCreate(grid->GetCellBackgroundColour(row, col).Red(),
       grid->GetCellBackgroundColour(row, col).Green(), grid->GetCellBackgroundColour(row, col).Blue());
}

void wxcGridSetCellTextFont (WXID id, WXID fontId, int row, int col)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-set-cell-text-font: could not find grid " << id << ".\n";
    return;
  }
  wxFont *font = (wxFont *) wxGetTypedObject (fontId, CLASSINFO(wxFont));
  if (!font)
  {
    ExtensionError << "Error in grid-set-cell-text-font: could not find font " << fontId << ".\n";
    return;
  }

  if (row == -1 && col == -1)
    grid->SetCellTextFont(font);
  else
    grid->SetCellTextFont(font, row, col);
}

void wxcGridSetColumnWidth (WXID id, int col, int width)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-set-column-width: could not find grid " << id << ".\n";
    return;
  }
  grid->SetColumnWidth(col, width);
}

int wxcGridGetColumnWidth (WXID id, int col)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-get-column-width: could not find grid " << id << ".\n";
    return 0;
  }
  return grid->GetColumnWidth(col);
}

void wxcGridSetRowHeight (WXID id, int row, int height)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-set-row-height: could not find grid " << id << ".\n";
    return;
  }
  grid->SetRowHeight(row, height);
}

int wxcGridGetRowHeight (WXID id, int row)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-get-row-height: could not find grid " << id << ".\n";
    return 0;
  }
  return grid->GetRowHeight(row);
}

void wxcGridSetLabelSize (WXID id, int orientation, int sz)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-set-label-size: could not find grid " << id << ".\n";
    return;
  }
  grid->SetLabelSize(orientation, sz);
}

void wxcGridSetLabelSize (WXID id, char *orientation, int sz)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-set-label-size: could not find grid " << id << ".\n";
    return;
  }
  int orient;
  wxString str(orientation);
  if (str == "wxHORIZONTAL")
    orient = wxHORIZONTAL;
  else
    orient = wxVERTICAL;
  grid->SetLabelSize(orient, sz);
}

int wxcGridGetLabelSize (WXID id, int orientation)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-get-label-size: could not find grid " << id << ".\n";
    return 0;
  }
  return grid->GetLabelSize(orientation);
}

int wxcGridGetLabelSize (WXID id, char *orientation)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-get-label-size: could not find grid " << id << ".\n";
    return 0;
  }
  int orient;
  wxString str(orientation);
  if (str == "wxHORIZONTAL")
    orient = wxHORIZONTAL;
  else
    orient = wxVERTICAL;
  return grid->GetLabelSize(orient);
}

void wxcGridSetLabelAlignment (WXID id, int orientation, int alignment)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-set-label-alignment: could not find grid " << id << ".\n";
    return;
  }
  grid->SetLabelAlignment(orientation, alignment);
}

int wxcGridGetLabelAlignment (WXID id, int orientation)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-get-label-alignment: could not find grid " << id << ".\n";
    return 0;
  }
  return grid->GetLabelAlignment(orientation);
}

void wxcGridSetLabelAlignment (WXID id, char *orientation, char *alignment)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-set-label-alignment: could not find grid " << id << ".\n";
    return;
  }
  int flag2 = 0;
  wxString str(alignment);
  if (str == "wxLEFT")
    flag2 = wxLEFT;
  else if (str == "wxRIGHT")
    flag2 = wxRIGHT;
  else if (str == "wxCENTRE")
    flag2 = wxCENTRE;
  else
  {
    ExtensionError << "Error in grid-set-cell-alignment: alignment should be one of wxLEFT, wxRIGHT, wxCENTRE.\n";
    return;
  }
  int orient;
  wxString str2(orientation);
  if (str2 == "wxHORIZONTAL")
    orient = wxHORIZONTAL;
  else
    orient = wxVERTICAL;
  
  grid->SetLabelAlignment(orient, flag2);
}

char *wxcGridGetLabelAlignment (WXID id, char *orientation)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-get-label-alignment: could not find grid " << id << ".\n";
    return NULL;
  }
  int orient;
  wxString str2(orientation);
  if (str2 == "wxHORIZONTAL")
    orient = wxHORIZONTAL;
  else
    orient = wxVERTICAL;
  
  int align = grid->GetLabelAlignment(orient);
  switch (align)
  {
    case wxRIGHT:
      return "wxRIGHT";
    case wxCENTRE:
      return "wxCENTRE";
    case wxLEFT:
    default:
      return "wxLEFT";
  }
  return NULL;
}

void wxcGridSetLabelValue (WXID id, int orientation, char *value, int pos)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-set-label-value: could not find grid " << id << ".\n";
    return;
  }
  grid->SetLabelValue(orientation, value, pos);
}

void wxcGridSetLabelValue (WXID id, char *orientation, char *value, int pos)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-set-label-value: could not find grid " << id << ".\n";
    return;
  }
  int orient;
  wxString str2(orientation);
  if (str2 == "wxHORIZONTAL")
    orient = wxHORIZONTAL;
  else
    orient = wxVERTICAL;
  grid->SetLabelValue(orient, value, pos);
}

char *wxcGridGetLabelValue (WXID id, int orientation, int pos)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-get-label-value: could not find grid " << id << ".\n";
    return NULL;
  }
  return grid->GetLabelValue(orientation, pos).GetData();
}

char *wxcGridGetLabelValue (WXID id, char *orientation, int pos)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-get-label-value: could not find grid " << id << ".\n";
    return NULL;
  }
  int orient;
  wxString str2(orientation);
  if (str2 == "wxHORIZONTAL")
    orient = wxHORIZONTAL;
  else
    orient = wxVERTICAL;
  return grid->GetLabelValue(orient, pos).GetData();
}

void wxcGridSetLabelTextColour (WXID id, WXCOLOUR colour)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-set-text-colour: could not find grid " << id << ".\n";
    return;
  }
  wxColour colour1((unsigned char)wxcColourRed(colour), (unsigned char)wxcColourGreen(colour), (unsigned char)wxcColourBlue(colour));
  grid->SetLabelTextColour(colour1);
}

WXCOLOUR wxcGridGetLabelTextColour (WXID id)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-get-text-colour: could not find grid " << id << ".\n";
    return 0;
  }
  return wxcColourCreate(grid->GetLabelTextColour().Red(), grid->GetLabelTextColour().Green(), grid->GetLabelTextColour().Blue());
}

void wxcGridSetLabelBackgroundColour (WXID id, WXCOLOUR colour)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-set-background-colour: could not find grid " << id << ".\n";
    return;
  }
  wxColour colour1((unsigned char)wxcColourRed(colour), (unsigned char)wxcColourGreen(colour), (unsigned char)wxcColourBlue(colour));
  grid->SetLabelBackgroundColour(colour1);
}

WXCOLOUR wxcGridGetLabelBackgroundColour (WXID id)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-get-background-colour: could not find grid " << id << ".\n";
    return 0;
  }
  return wxcColourCreate(grid->GetLabelBackgroundColour().Red(), grid->GetLabelBackgroundColour().Green(), grid->GetLabelBackgroundColour().Blue());
}

void wxcGridSetLabelTextFont (WXID id, WXID fontId)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-set-label-text-font: could not find grid " << id << ".\n";
    return;
  }
  wxFont *font = (wxFont *) wxGetTypedObject (fontId, CLASSINFO(wxFont));
  if (!font)
  {
    ExtensionError << "Error in grid-set-label-text-font: could not find font " << fontId << ".\n";
    return;
  }

  grid->SetLabelTextFont(font);
}

int wxcGridGetCursorRow (WXID id)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-get-cursor-row: could not find grid " << id << ".\n";
    return 0;
  }
  return grid->GetCursorRow();
}

int wxcGridGetCursorColumn (WXID id)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-get-cursor-column: could not find grid " << id << ".\n";
    return 0;
  }
  return grid->GetCursorColumn();
}

void wxcGridSetGridCursor (WXID id, int row, int col)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-set-grid-cursor: could not find grid " << id << ".\n";
    return;
  }
  grid->SetGridCursor(row, col);
}

int wxcGridGetRows (WXID id)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-get-rows: could not find grid " << id << ".\n";
    return 0;
  }
  return grid->GetRows();
}

int wxcGridGetCols (WXID id)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-get-cols: could not find grid " << id << ".\n";
    return 0;
  }
  return grid->GetCols();
}

int wxcGridGetScrollPosX (WXID id)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-get-scroll-pos-x: could not find grid " << id << ".\n";
    return 0;
  }
  return grid->GetScrollPosX();
}

int wxcGridGetScrollPosY (WXID id)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-get-scroll-pos-y: could not find grid " << id << ".\n";
    return 0;
  }
  return grid->GetScrollPosY();
}

void wxcGridSetEditable (WXID id, Bool editable)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-set-editable: could not find grid " << id << ".\n";
    return;
  }
  grid->SetEditable(editable);
}

Bool wxcGridGetEditable (WXID id)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-get-editable: could not find grid " << id << ".\n";
    return FALSE;
  }
  return grid->GetEditable();
}

void wxcGridSetDividerPen (WXID id, WXID penId)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-set-divider-pen: could not find grid " << id << ".\n";
    return;
  }
  // penId is allowed to be 0, to mean 'no dividers'
  wxPen *pen = NULL;
  if (penId != 0)
  {
    pen = (wxPen *) wxGetTypedObject (penId, CLASSINFO(wxPen));
    if (!pen)
    {
      ExtensionError << "Error in grid-set-divider-pen: could not find pen " << penId << ".\n";
      return;
    }
  }
  grid->SetDividerPen(pen);
}

void wxcGridAdjustScrollbars (WXID id)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-adjust-scrollbars: could not find grid " << id << ".\n";
    return;
  }
  grid->AdjustScrollbars();
}

void wxcGridUpdateDimensions (WXID id)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-update-dimensions: could not find grid " << id << ".\n";
    return;
  }
  grid->UpdateDimensions();
}

void wxcGridOnActivate (WXID id, Bool active)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-on-activate: could not find grid " << id << ".\n";
    return;
  }
  ((wxGenericGrid *)grid)->wxGenericGrid::OnActivate(active);
}

void wxcGridOnPaint (WXID id)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-on-paint: could not find grid " << id << ".\n";
    return;
  }
  ((wxGenericGrid *)grid)->wxGenericGrid::OnPaint();
}

void wxcGridOnSize (WXID id, int w, int h)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-on-size: could not find grid " << id << ".\n";
    return;
  }
  ((wxGenericGrid *)grid)->wxGenericGrid::OnSize(w, h);
}

long wxcGridGetTextItem(WXID id)
{
  wxGrid *grid = (wxGrid *) wxGetTypedObject (id, CLASSINFO(wxGenericGrid));
  if (!grid)
  {
    ExtensionError << "Error in grid-get-text-item: could not find grid " << id << ".\n";
    return -1;
  }
  return (long)grid->GetTextItem();
}

#endif
